/**
 * \file gre/layerview.h
 * \brief GRE::LAYERVIEW class definition
 * 
 * \if NODOC
 * $Id: layerview.h_v 1.36 2012/05/03 21:43:53 mju Exp $
 *
 * $Log: layerview.h_v $
 * Revision 1.36  2012/05/03 21:43:53  mju
 * Add estimatedRangeZ and method to get.
 *
 * Revision 1.35  2011/10/20 17:13:54  mju
 * Remove clearQueuedItems as did nothing.
 *
 * Revision 1.34  2011/03/21 16:44:54  mju
 * Remove virtual tiling support, make dtor virtual.
 *
 * Revision 1.33  2010/08/19 17:40:18  mju
 * Use elemnum type.
 *
 * Revision 1.32  2010/04/23 16:54:25  mju
 * Use unsignedset.
 *
 * Revision 1.31  2010/04/16 21:19:26  mju
 * Remove elemSelectionSet stuff as never used.
 *
 * Revision 1.30  2008/09/02 17:12:27  mju
 * Add clearQueuedItems.
 *
 * Revision 1.28  2008/06/16 20:48:11  mju
 * Add queueJobRender...
 *
 * Revision 1.25  2008/05/15 17:54:32  mju
 * More layersection changes.
 *
 * Revision 1.23  2008/04/15 13:33:51  mju
 * Add isAttachedToView.
 *
 * Revision 1.22  2008/04/11 16:19:57  mju
 * Make overlay methods const.
 *
 * Revision 1.17  2008/03/18 12:37:04  mju
 * Make critsec mutable.
 *
 * Revision 1.16  2008/03/17 21:33:09  mju
 * Use layer section list.
 *
 * Revision 1.15  2008/03/10 14:35:21  mju
 * Remove unimplemented methods.
 *
 * Revision 1.14  2007/01/16 18:57:27  mju
 * Add transChanged member.
 *
 * Revision 1.13  2006/09/14 19:47:26  mju
 * Fwd declare bitset.
 *
 * Revision 1.12  2006/09/07 21:48:19  mju
 * Remove hasSurfaceNulls.
 *
 * Revision 1.11  2006/09/05 18:12:39  mju
 * Add m_prevObjRectFull.
 *
 * Revision 1.10  2005/10/19 19:58:20  mju
 * Use gre namespace.
 *
 * Revision 1.9  2005/10/14 17:30:36  mju
 * GRE namespace.
 *
 * Revision 1.8  2005/05/24 21:29:36  mju
 * Make cliprect optional for overlayBufferIfRendered.
 *
 * Revision 1.7  2005/05/24 17:55:04  mju
 * Add OverlayIfRendered.
 *
 * Revision 1.6  2005/04/29 17:49:36  mju
 * Support rendertarget.
 *
 * Revision 1.5  2005/04/27 15:31:30  mju
 * Add docs for getElemDrawnSet.
 *
 * Revision 1.4  2005/04/22 17:35:44  mju
 * Use renderbuffer class.
 *
 * Revision 1.3  2005/03/22 21:59:50  mju
 * Reset overlay rendering and clear label placement in resetRendered.
 *
 * Revision 1.2  2005/03/01 14:21:31  mju
 * Add layerview_geometric to manage drawn sets.
 *
 * Revision 1.1  2005/02/25 18:54:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LAYERVIEW_H
#define	INC_GRE_LAYERVIEW_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_GRE_LABELPLACEMENT_H
	#include <gre/labelplacement.h>
#endif

#ifndef	INC_MI32_RENDERBUFFER_H
	#include <mi32/renderbuffer.h>
#endif

#ifndef	INC_MI32_TRANS2DC_H
	#include <mi32/trans2dc.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
	#include <mi32/critsec.h>
#endif


class UNSIGNEDSET;
namespace MITHREAD {
	class QUEUE;
	}


namespace GRE {

//===================================================================================================================

//! Container for layer information retained for each view.
class LAYERVIEW {
	public:

		//! Construct with layer and view.
		LAYERVIEW (
			GRE::LAYER *layer,
			GRE::VIEW *view
			);

		//! Construct with layer, view and object/device transformation.
		//! Typically for internal GRE use only.
		LAYERVIEW (
			GRE::LAYER *layer,
			GRE::VIEW *view,
			const TRANS2D_MAPGEN& TransObjectToDevice,
			INT32 DevSizeX,
			INT32 DevSizeY
			);

		//! Destructor.
		virtual ~LAYERVIEW ();

		//! Attach rendered section.
		void AttachSection (
			GRE::LAYERSECTION *& pLayerSection,		//!< Section to attach, will be returned as NULL
			bool NotifyView = true
			);

		//! Check if existing renderings are still valid.
		void CheckRendered ();

		//! Clear label placement if any.
		void ClearLabelPlacement ();

		//! Compute all transformations.
		ERRVALUE ComputeTrans ();

		//! Detach specified layer section if exists.
		//! @return true if section was previously attached, false if not
		bool DetachSection (
			const LAYERSECTION *pLayerSection	//!< Section to detach
			);

		//! Draw single element to specified rendering buffer.
		ERRVALUE DrawElementToBuffer (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum,
			GRE::RENDERTARGET content,
			GRE::DRAWFLAGS drawflags
			);

		//! Draw single element to specified rendering device.
		ERRVALUE DrawElementToDevice (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum,
			MGD::DEVICE *pDevice,
			GRE::DRAWFLAGS drawflags
			);

		//! Draw set of elements to specified rendering buffer.
		ERRVALUE DrawElementsToBuffer (
			ELEMTYPE ElemType,
			const UNSIGNEDSET *pElemSet,
			GRE::RENDERTARGET content,
			GRE::DRAWFLAGS drawflags
			);

		//! Draw set of elements to specified rendering device.
		ERRVALUE DrawElementsToDevice (
			ELEMTYPE ElemType,
			const UNSIGNEDSET *pElemSet,
			MGD::DEVICE *pDevice,
			GRE::DRAWFLAGS drawflags
			);

		//! Draw highlighted elements to corresponding rendering buffer.
		ERRVALUE DrawHighlightedToBuffer (
			GRE::DRAWFLAGS drawflags
			);

		//! Get depth scaling, multiplied by depth to get drawing scale.
		double Get3dDepthScale (
			) const { return (m_3dDepthScale); }

		//! Get increment in layer relative to once cell in surface layer.
		double Get3dSurfaceIncrement (
			) const { return (m_3dSurfaceIncrement); }

		//! Get 3D viewpoint.
		const GRE::VIEWPOINT3D* Get3dViewPoint (
			) const { return (m_pViewPoint3D); }

		//! Get 3D Z scaling.
		double Get3dZScale (
			) const { return (m_3dZScale); }

		//! Get design scale.
		double GetDesignScale (
			) const { return (m_DesignScale); }

		//! Get device rectangle used for rendering.
		//! This is a nominal rectangle based on layer extents and may not account for
		//! symbology and marginalia.  Returned rectangle may not be suitable for clipping
		//! during drawing, but will be suitable for selection of source object elements.
		const LRECT2D& GetDeviceRectUsed (
			) const { return (m_CurDevRectUsed); }

		//! Get set for elements drawn in view (const).
		//! @return NULL if no elements of specified type supported by layer.
		//! The set returned by this method only contains the elements which are actually visible in the view.
		//! Normally when checking for highlighted elements, the GRE::LAYER::GetElemHighlightSet method should be used.
		const UNSIGNEDSET* GetElemDrawnSet (
			ELEMTYPE ElemType,										//!< Element type
			bool UseHighlighted										//!< True to return set for visible elements drawn as 'highlighted'
			) const;

		//! Get set for elements drawn in view (non-const).
		//! @return NULL if no elements of specified type supported by layer.
		//! The set returned by this method should only contain the elements which are actually visible in the view.
		//! Normally when checking for highlighted elements, the GRE::LAYER::GetElemHighlightSet method should be used.
		UNSIGNEDSET* GetElemDrawnSet (
			ELEMTYPE ElemType,										//!< Element type
			bool UseHighlighted										//!< True to return set for visible elements drawn as 'highlighted'
			);

		//! Get estimated Z range for layer.
		ERRVALUE GetEstimatedRangeZ (
			DOUBLE_RANGE& range										//!< Estimated Z range for this associated layer returned.
			);

		//! Get pointer to label placement instance.
		//! @return LABELPLACEMENT pointer or NULL if not supported.
		LABELPLACEMENT* GetLabelPlacement (
			bool ReturnNullIfEmpty = true							//!< Return NULL pointer if no labels
			);

		//! Get associated GRE::LAYER.
		GRE::LAYER* GetLayer (
			) const { return (m_layer); }

		//! Get LAYERSECTION pointer.
		//! @return Pointer to LAYERSECTION.
		const LAYERSECTION* GetSection (
			) const { return (m_pSectionUntiled); }

		//! Get LAYERSECTION pointer (non-const)
		//! @return Pointer to LAYERSECTION.
		LAYERSECTION* GetSection (
			) { return (m_pSectionUntiled); }

		//! Get surface layer for 3D draping.
		GRE::LAYER_SURFACE* GetSurfaceLayer (
			) const { return (m_pSurfaceLayer); }

		//! Get transformation from layer to device coordinates.
		const TRANS2D_MAPGEN& GetTransLayerDevice (
			) const { return (m_TransLayerDevice); }

		//! Get transformation from 'layer' to 'surface' coordinates.
		const TRANS2D_MAPGEN& GetTransLayerSurface (
			) const { return (m_TransLayerSurface); }

		//! Get transformation from layer to view coordinates.
		const TRANS2D_MAPGEN& GetTransLayerView (
			) const { return (m_TransLayerView); }

		//! Get associated GRE::VIEW.
		GRE::VIEW* GetView () const
			{ return (m_view); }

		//! Determined if has rendered specified content for all portions visible in view.
		bool HasRendered (
			RENDERTARGET content = RENDERTARGET_LayerAll
			) const;

		//! Determine if drawing in 3D.
		bool Is3D () const
			{ return (m_Is3D); }

		//! Determine if drawing 3D in stereo.
		bool Is3dStereo () const;

		//! Determine if instance is attached to and owned by VIEW.
		bool IsAttachedToView () const;

		//! Determine if surface (layer or flat) is being used.
		bool IsSurfaceUsed (
			) const { return (m_IsSurfaceUsed); }

		//! Determine if layer overlaps view device display area.
		bool IsLayerOverlappedDevice (
			) const { return (m_LayerOverlapsDevice); }

		//! Overlay rendering buffer on device if it has been rendered.
		//! @return true if did overlay, false if not.
		bool OverlayBufferIfRendered (
			GRE::RENDERTARGET content,								//!< Content type
			MGD::DEVICE *pDevTarget,								//!< Target device
			const LRECT2D *pClipRect = 0							//!< Optional clip rectangle
			) const;

		//! Queue job for rendering full view extent.
		ERRVALUE QueueJobRenderFull (
			MITHREAD::QUEUE& Queue									//! Queue to place job in
			);

		//! Reset all 'drawn element' sets to have no entries.
		void ResetAllDrawnSets ();

		//! Reset rendered status so redraws.
		void ResetRendered (
			GRE::RENDERTARGET target = GRE::RENDERTARGET_LayerAll
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GRE::LAYER *m_layer;
		GRE::VIEW *m_view;
		LAYERSECTION *m_pSectionUntiled;
		TRANS2D_MAPGEN m_TransLayerView;
		TRANS2D_MAPGEN m_TransLayerDevice;
		TRANS2D_MAPGEN m_TransLayerSurface;
		GRE::VIEWPOINT3D *m_pViewPoint3D;			// 3D viewpoint
		GRE::LAYER_SURFACE *m_pSurfaceLayer;		// Surface layer in 3D
		DRECT2D m_NewTgtRectFull;						// New full rectangle in target device coordinates, based on transformation
		DRECT2D m_CurTgtRectFull;						// Current full rectangle in target device coordinates used by layer
		DRECT2D m_PrevObjRectFull;						// Previous full object rectangle (for detecting extent changed)
		LRECT2D m_NewDevRectUsed;						// New clipped rectangle on device used, based on transformation
		LRECT2D m_CurDevRectUsed;						// Current clipped rectangle on device used
		double m_DesignScale;							// Design scale of layout/group for relative size
		double m_3dSurfaceIncrement;					// Increment in layer corresponding to one cell in surface raster
		double m_3dDepthScale;							// Map scale factor for 3D (multiply by depth to get actual scale)
		double m_3dZScale;
		DOUBLE_RANGE m_EstimatedRangeZ;
		bool m_Is3D;										// Set if drawing in 3D
		bool m_IsSurfaceUsed;							// Use surface layer for Z value
		bool m_LayerOverlapsDevice;
		bool m_TransChanged;
		mutable MITHREAD::CRITICALSECTION m_CritSec;

		LAYERVIEW (const LAYERVIEW&);
		LAYERVIEW& operator= (const LAYERVIEW&);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYERVIEW_H
