/**
 * \file mgui/layout.h 
 * \brief MGUI::LAYOUT class definitions.
 *
 * \if NODOC
 * $Id: layout.h_v 1.62 2011/11/22 21:33:58 mju Exp $
 *
 * $Log: layout.h_v $
 * Revision 1.62  2011/11/22 21:33:58  mju
 * Fix dox dependencies.
 *
 * Revision 1.61  2011/11/08 15:31:09  mju
 * Remove all mfc stuff.
 *
 * Revision 1.60  2011/11/08 14:49:59  mju
 * Move etslayout stuff to own hdr.
 *
 * Revision 1.59  2011/06/21 14:43:25  mju
 * Doc cleanup.
 *
 * Revision 1.57  2011/04/08 18:08:49  mju
 * Add computeTextSize using textid.
 *
 * Revision 1.56  2010/06/15 17:32:47  mju
 * Add pane_base::get/setDimensions.
 *
 * Revision 1.55  2010/03/22 19:48:24  mju
 * Add pane_base::setMinimumSize.
 *
 * Revision 1.54  2010/03/12 21:51:21  mju
 * Add pane_base::invalidate.
 *
 * Revision 1.53  2010/02/24 22:09:35  mju
 * Add layout_pane_sash.
 *
 * Revision 1.52  2009/09/24 17:21:28  mju
 * Add getTextMaxHeight.
 *
 * Revision 1.51  2007/10/15 17:17:00  mju
 * Add isVisible.
 *
 * Revision 1.50  2007/07/12 17:44:03  mju
 * Add get/setBackgroundColor for pane_base.
 *
 * Revision 1.48  2007/02/02 15:52:12  mju
 * Add page create() variant with delegate.
 *
 * Revision 1.32  2005/05/24 02:54:57  dfriberg
 * Added ComputeNumFittingCharacters
 *
 * Revision 1.28  2004/07/26 15:10:28  mju
 * Add setminimumsize for equalizer.
 *
 * Revision 1.11  2002/08/08 22:04:03  mju
 * Add book/page methods for determine/setting active page and enabling/disabling.
 *
 * Revision 1.6  2001/12/18 16:34:32  mju
 * Add PANE_BASE::UpdateLayout().
 *
 * Revision 1.5  2001/12/17 18:51:46  mju
 * Add EQUALIZER::SetEqualize method.
 *
 * Revision 1.1  2001/12/13 16:59:56  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_LAYOUT_H
#define	INC_MGUI_LAYOUT_H

#ifndef	INC_MGUI_BASE_H
#include <mgui/base.h>
#endif

#ifndef INC_MI32_XTABPAGE_H
#include <mi32/xtabpage.h>
#endif

#ifndef INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MISTRING;
struct COLOR;
class SML_GUI_INTERFACE_BASE;
namespace MGUI {
	class LAYOUT_PANE_ROOT;
	class LAYOUT_PANE_SHELL;
	class LAYOUT_PAGE;
	}
#endif


namespace MGUI {

//=====================================================================================================================

//! Layout pane base class, not intended to be used directly.
class LAYOUT_PANE_BASE {
	public:

		//! Constructor.
		LAYOUT_PANE_BASE (
			):
			m_PaneWidget(0)
			{ }

		//! Destructor.
		virtual ~LAYOUT_PANE_BASE () = 0;

		//! Add automatically expanding whitespace item.
		void AddSpaceExpanding ();

		//! Add whitespace item of fixed size.
		void AddSpaceFixed (
			int size
			);

		//! Compute the number of characters of the string that fit in the given width.
		int ComputeNumFittingCharacters (
			const MISTRING& str,
			int width
			) const;

		//! Compute typical size for given text width in characters.
		MGUI::POINT ComputeTextSize (
			int width								//!< Width in characters
			) const;

		//! Compute size of given label text from MISTRING.
		MGUI::POINT ComputeTextSize (
			const MISTRING& str					//!< String
			) const;

		//! Compute size of given label TEXTID.
		MGUI::POINT ComputeTextSize (
			TEXTID textid							//!< Text ID
			) const;

		//! Compute size of given label text as MIUNICODE string.
		MGUI::POINT ComputeTextSize (
			const MIUNICODE *str					//!< String
			) const;

		//! Get 'background' color.
		void GetBackgroundColor (
			COLOR& color
			) const;

		//! Get pane dimensions.
		MGUI::POINT GetDimensions () const;

		//! Get suitable parent for dialog launched from pane.
		MDLGPARENT GetDlgParent (
			) const;

		//! Get maximum text height.
		int GetTextMaxHeight ();

		//! Get pane widget.
		Widget GetWidget (
			) const { return (m_PaneWidget); }

		//! Invalidate pane, forcing redraw.
		void Invalidate (
			bool erase = true
			);

		//! Determine if pane has been created.
		bool IsCreated () const
			{ return (m_PaneWidget != 0); }

		//! Determine if pane is visible.
		//! Does not check if pane is contained in a non-visible parent.
		bool IsVisible (
			) const;

		//! Set 'background' color.
		void SetBackgroundColor (
			const COLOR& color
			);

		//! Set pane dimensions.
		void SetDimensions (
			MGUI::POINT dim
			);

		//! Set 'enabled' status for all controls in pane or children therof.
		void SetEnabled (
			bool enabled
			);

		//! Set minimum size for pane.
		void SetMinimumSize (
			int width,						//!< Minimum width in pixels, -1 to leave unchanged
			int height						//!< Minimum height in pixels, -1 to leave unchanged
			);

		//! Set visibility for all controls in pane or children thereof.
		void SetVisible (
			bool visible
			);

		//! Force update of layout.
		//! This may be necessary if the layout is changed after the dialog/window is initially created.
		void UpdateLayout (
			) { }

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Function private to SML implemention of MGUI classes
		//! Used to prevent the need for ugly multiple inheritance.
		virtual SML_GUI_INTERFACE_BASE* GetSMLInterface (
			);
		#endif

	protected:

		void SetPaneWidget (Widget PaneWidget);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		Widget m_PaneWidget;

		static void CB_Destroy (Widget, LAYOUT_PANE_BASE*, void*);

		LAYOUT_PANE_BASE (const LAYOUT_PANE_BASE&);
		LAYOUT_PANE_BASE& operator= (const LAYOUT_PANE_BASE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane for use as root of dialog, window or page.
class LAYOUT_PANE_ROOT : public MGUI::LAYOUT_PANE_BASE {
	public:

		//! Constructor.
		LAYOUT_PANE_ROOT (
			) { }

		//! Destructor.
		virtual ~LAYOUT_PANE_ROOT ();

		//! Initialize called internally by OnInitDialog() or similar (X version).
		void Init (
			Widget TopPaneWidget							//!< Topmost "Pane" widget
			) { SetPaneWidget(TopPaneWidget); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LAYOUT_PANE_ROOT (const LAYOUT_PANE_ROOT&);
		LAYOUT_PANE_ROOT& operator= (const LAYOUT_PANE_ROOT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane for a shell.
class LAYOUT_PANE_SHELL : public MGUI::LAYOUT_PANE_ROOT {
	public:

		//! Constructor.
		LAYOUT_PANE_SHELL (
			) { }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LAYOUT_PANE_SHELL (const LAYOUT_PANE_SHELL&);
		LAYOUT_PANE_SHELL& operator= (const LAYOUT_PANE_SHELL&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane intended for general use.
class LAYOUT_PANE : public MGUI::LAYOUT_PANE_BASE {
	public:

		//! Constructor, initializes only, does not create actual pane.
		LAYOUT_PANE (
			) { }

		//! Destructor.
		virtual ~LAYOUT_PANE (
			) { }

		//! Create the actual pane.
		void Create (
			MGUI::LAYOUT_PANE_BASE& pane,				//!< 'Parent' pane
			MGUI::LAYOUT_ORIENTATION orientation,	//!< Pane orientation
			MGUI::LAYOUT_SIZEALIGN sizealign,		//!< Sizing/alignment
			int ChildSpacing = 4,						//!< Spacing between 'children' of pane
			int ExtraBorder = 0							//!< Extra border around pane, in addition to ChildSpacing
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LAYOUT_PANE (const LAYOUT_PANE&);
		LAYOUT_PANE& operator= (const LAYOUT_PANE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane to be used as "sash" for sizing adjacent panes.
//! A sash control allows the user to resize the two adjacent panes/controls by dragging.
//! Any space not covered by other controls will be usable for adjustment.  For this reason it
//! is typical that the center part of the pane remain empty and controls be added at the ends.
class LAYOUT_PANE_SASH : public MGUI::LAYOUT_PANE_BASE {
	public:

		//! Constructor, initializes only, does not create actual pane.
		LAYOUT_PANE_SASH (
			);

		//! Destructor.
		virtual ~LAYOUT_PANE_SASH (
			);

		//! Create the pane.
		//! The orientation will be determined based on the parent pane's orientation.
		void Create (
			MGUI::LAYOUT_PANE_BASE& pane,				//!< 'Parent' pane
			int Thickness = 4,							//!< Sash thickness, will expand if needed to contain added controls
			int ChildSpacing = 4,						//!< Spacing between 'children' of pane
			int ExtraBorder = 0							//!< Extra border around pane, in addition to ChildSpacing
			);

		//! Set delegate to call after sash moved.
		//! Typcally used to record settings.
		void SetDelegateOnMoved (
			DELEGATE_VOID_NOPARMS delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		LAYOUT_PANE_SASH (const LAYOUT_PANE_SASH&);
		LAYOUT_PANE_SASH& operator= (const LAYOUT_PANE_SASH&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane for "book" of tabbed pages.
//! First Initialize the book, then create the pages and finally create the book itself.
class LAYOUT_BOOK : public MGUI::LAYOUT_PANE_BASE {
	public:

		//! Constructor.
		LAYOUT_BOOK (
			);

		//! Create the book after page creation.
		void Create (
			);

		//! Get index to 'active' page.
		//! @return Zero-based index to active page, -1 if none created.
		int GetActiveIndex (
			) const;

		MGUI::LAYOUT_PANE_BASE* GetParentPane (
			) { return (m_pParentPane); }
		
		//! Initialize prior to page creation.
		void Initialize (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< 'Parent' pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Set active page by index.
		void SetActiveIndex (
			int index										//!< Zero-based page index
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<MxTABPAGEITEM> m_TabPageItems;
		Widget m_TabPage;
		MGUI::LAYOUT_PANE_BASE *m_pParentPane;
		LAYOUT_SIZEALIGN m_sizealign;

		static void TABCB_Create (Widget, int, void*);
		static void TABCB_Activate (Widget, int, void*);
		static void TABCB_Deactivate (Widget, int, void*);
		
		LAYOUT_BOOK (const LAYOUT_BOOK&);
		LAYOUT_BOOK& operator= (const LAYOUT_BOOK&);

		friend class LAYOUT_PAGE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Layout pane for use as tab page.
class LAYOUT_PAGE : public MGUI::LAYOUT_PANE_ROOT {
	public:

		typedef fastdelegate::FastDelegate<void(LAYOUT_PAGE&)> DELEGATE;

		//! Constructor.
		LAYOUT_PAGE (
			);

		//! Destructor.
		virtual ~LAYOUT_PAGE (
			);

		//! Create page with tab label from MISTRING.
		//! @return Page index.
		int Create (
			MGUI::LAYOUT_BOOK& book,					//!< Book to add page to
			const MISTRING& label,						//!< Label string
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical
			);

		//! Create page using delegate.
		//! @return Page index.
		//! Should not be used if OnInitPage is overridden, as delegate will be ignored.
		int Create (
			MGUI::LAYOUT_BOOK& book,					//!< Book to add page to
			const MISTRING& label,						//!< Label string
			DELEGATE DelegateOnInitPage,				//!< Delegate to call to initialize page
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			bool IgnoreNonVisible = false,			//!< Ignore non-visible items when doing layout
			bool AddSpaceAtEnd = true					//!< Add "expanding" space at end (usually bottom) of page
			);

		//! Determine if this is the 'active' page.
		bool IsActive (
			) const;

		//! Set delegate to call for page initialization.
		//! If used, must be called before Create
		void SetDelegateOnInitPage (
			DELEGATE delegate
			) { m_DelegateOnInitPage = delegate; }

		//! Set delegate to call when page made active.
		void SetDelegateOnSetActive (
			DELEGATE delegate
			) { m_DelegateOnSetActive = delegate; }

		//! Set delegate to call when page made inactive.
		void SetDelegateOnSetInactive (
			DELEGATE delegate
			) { m_DelegateOnSetInactive = delegate; }

		//! Set whether page is 'enabled' or not.
		void SetEnabled (
			bool enabled = true
			);

		//! Set to ignore nonvisible items when doing layout.
		//! This must be called before Create() and is usually done in the subclass constructor.
		void SetIgnoreNonVisible (
			bool IgnoreNonVisible = true
			) { m_IgnoreNonVisible = IgnoreNonVisible; }

		//! Set whether to not automatically add "expanding space" at end (bottom usually) of page.
		//! This must be called before Create() and is usually done in the subclass constructor.
		//! Default behavior is to add expanding space.
		void SetNoSpaceExpanding (
			bool NoSpaceExpanding = true
			) { m_NoSpaceExpanding = NoSpaceExpanding; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_index;
		LAYOUT_BOOK *m_pBook;
		MGUI::LAYOUT_ORIENTATION m_Orientation;
		bool m_IgnoreNonVisible;
		bool m_NoSpaceExpanding;
		DELEGATE m_DelegateOnInitPage;
		DELEGATE m_DelegateOnSetActive;
		DELEGATE m_DelegateOnSetInactive;

		LAYOUT_PAGE (const LAYOUT_PAGE&);
		LAYOUT_PAGE& operator= (const LAYOUT_PAGE&);

		friend class LAYOUT_BOOK;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Create page contents.
		//! Default implementation calls delegate.
		//! If not overridden then delegate must be specified.
		virtual void OnInitPage (
			);

		//! Called when page is made active.
		virtual void v_OnSetActive (
			);

		//! Called when page is made inactive.
		virtual void v_OnSetInactive (
			);

	};

//=====================================================================================================================

//! Automatically equalize width and/or height of controls for alignment.
class LAYOUT_EQUALIZER {
	public:

		//! Dimension to equalize.
		enum EQUALIZE {
			EQUALIZE_Width =		1,
			EQUALIZE_Height =		2,
			EQUALIZE_Both =		3
			};

		//! Constructor.
		LAYOUT_EQUALIZER (
			EQUALIZE equalize = EQUALIZE_Width	//!< What dimension(s) to equalize
			);

		//! Destructor.
		~LAYOUT_EQUALIZER (
			);

		//! Add MGUI::CTRL to be equalized.
		void Add (
			MGUI::CTRL& control
			);

		//! Perform equalization.
		//! This is automatically done by destructor if not done previously.
		void Equalize ();

		//! Set dimensions to equalize.
		void SetEqualize (
			EQUALIZE equalize
			) { m_equalize = equalize; }

		void SetMinimumSize (
			int minwidth,
			int minheight = 0
			) { m_minwidth = minwidth; m_minheight = minheight; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class WLIST;

		EQUALIZE m_equalize;
		WLIST *m_plist;
		int m_minwidth;
		int m_minheight;

		LAYOUT_EQUALIZER (const LAYOUT_EQUALIZER&);
		LAYOUT_EQUALIZER& operator= (const LAYOUT_EQUALIZER&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End of MGUI namespace

#endif	// INC_MGUI_LAYOUT_H
