/** 
 * \file mgui/slider.h 
 * \brief Definitions for MGUI::SLIDER class.
 *
 * \if NODOC
 * $Id: slider.h_v 1.11 2011/11/09 15:52:59 mju Exp $
 *
 * $Log: slider.h_v $
 * Revision 1.11  2011/11/09 15:52:59  mju
 * Remove mfc stuff.
 *
 * Revision 1.9  2008/01/29 14:02:57  mju
 * Support logarithmic scaling.
 *
 * Revision 1.8  2008/01/21 17:16:00  mju
 * Change setStep to return bool.
 *
 * Revision 1.7  2008/01/21 16:33:57  mju
 * Use double for min/max/range on ctrl_slider.
 * Use separate method for setStep and remove parm from create.
 * Add editwidth parm to form::create.
 *
 * Revision 1.4  2007/10/18 17:53:34  bcolombe
 * Added SetPlaces to FORM_SLIDER
 *
 * Revision 1.2  2007/06/13 22:16:35  honerkamp
 * Add a SetRange function
 *
 * Revision 1.1  2007/06/13 18:06:27  honerkamp
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_SLIDER_H
#define  INC_MGUI_SLIDER_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Slider bar control.
//! A slider control provides a way to graphically adjust a numeric value.  The actual accuracy of the associated
//! value is limited to the displayed length of the slider "trough".
class CTRL_SLIDER : public MGUI::CTRL {
	public:

		enum STYLE {
			STYLE_Horizontal =			0x0001,	//!< lays down horizontal
			STYLE_Vertical =				0x0002,	//!< stands up vertical
			STYLE_ReverseDirection =	0x0004,	//!< left or down would be positive
			STYLE_Logarithmic =			0x0008,	//!< Logarithmic scale
			};

		//! Constructor.
		CTRL_SLIDER ();

		//! Destructor.
		virtual ~CTRL_SLIDER  (
			);

		//! Create slider form
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			double dftvalue,								//!< Default value
			double min,										//!< Minimum value for the slider bar
			double max,										//!< Maximum value for the slider bar.  If stepsize is set needs to be min plus integral multiple of stepsize
			int length = 100,								//!< Length of the slider in pixels
			STYLE style = STYLE_Horizontal,			//!< Slider style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get current step setting.
		double GetStep (
			) const { return (m_step); }

		//! Get current slider value
		double GetValue (
			) const { return (m_value); }

		//! Set color for slider bar.
		void SetColorBar (
			const COLOR &barcolor
			);

		//! Set color for slider trough.
		void SetColorTrough (
			const COLOR &troughcolor
			); 

		//! Set delegate called after value gets changed via dragging and letting go of slider.
		void SetDelegateOnValueChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnValueChanged = delegate; }

		//! Called while user drags slider each time is moved, useful for dynamically updating something else
		void SetDelegateOnUserSliding (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnUserSliding = delegate; }

		//! Set new range for the slider
		void SetRange (
			double min,										//!< New minimum
			double max									   //!< New maximum
			);

		//! Set step for rounding.
		//! If non-zero, values will be rounded to integral multiple of 'step' from minimum.
		//! @return true if results in value change, false if not.
		bool SetStep (
			double step,									//!< Step for rounding
			bool notify = true							//!< If true call OnValueChanged delegate if actual value changed
			);
		
		//! Set value for the slider
		void SetValue (
			double value,									//!< Value to set
			bool notify = true							//!< If true call OnValueChanged delegate if actual value changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnValueChanged;
		DELEGATE_VOID_NOPARMS m_DelegateOnUserSliding;

		bool m_UseRawDirect;
		bool m_IsReversed;
		bool m_IsLogarithmic;
		double m_value;
		double m_min;
		double m_max;
		double m_step;
		double m_scaleraw;
		double m_ScaleMinLog;
		double m_ScaleMaxLog;

		#ifdef X_NATIVE
		static void X_Drag(Widget,CTRL_SLIDER*,void*);
		static void X_ValueChanged(Widget,CTRL_SLIDER*,void*);
		#endif

		double ComputeValue (int ctrlpos) const;
		void OnDrag (int ctrlpos);
		void OnValueChanged (int ctrlpos);
		void UpdateCtrl (bool setrange);

		CTRL_SLIDER (const CTRL_SLIDER&);
		CTRL_SLIDER& operator= (const CTRL_SLIDER&);
		#endif //dyoxygen output
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_SLIDER::STYLE)
#endif

//=====================================================================================================================

//! Form containing optional label, slider and numeric edit controls.
class FORM_SLIDER : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_SLIDER ();
		
		//! Destructor.
		virtual ~FORM_SLIDER  (
			);

		//! Create slider form with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			double dftvalue,								//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int sliderlength = 100,						//!< Length of the slider in pixels
			int editplaces = 0,							//!< Number of Decimal places to show in "edit" control
			int editwidth = 0,							//!< Width in "characters" of "edit" control, 0 to base on range and places
			CTRL_SLIDER::STYLE sliderstyle = CTRL_SLIDER::STYLE_Horizontal,			//!< Slider style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create slider form with a label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			double dftvalue,								//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int sliderlength = 100,						//!< Length of the slider in pixels
			int editplaces = 0,							//!< Number of Decimal places to show in "edit" control
			int editwidth = 0,							//!< Width in "characters" of "edit" control, 0 to base on range and places
			CTRL_SLIDER::STYLE sliderstyle = CTRL_SLIDER::STYLE_Horizontal,			//!< Slider style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create slider form with a label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			TEXTID label,									//!< Label text id
			double dftvalue,								//!< Default value
			double min,										//!< Minimum value
			double max,										//!< Maximum value
			int sliderlength = 100,						//!< Length of the slider in pixels
			int editplaces = 0,							//!< Number of Decimal places to show in "edit" control
			int editwidth = 0,							//!< Width in "characters" of "edit" control, 0 to base on range and places
			CTRL_SLIDER::STYLE sliderstyle = CTRL_SLIDER::STYLE_Horizontal,			//!< Slider style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Retrieve string "edit" control for form.
		CTRL_EDIT_NUMBER& GetEditCtrl (
			) { return (m_editctrl); }
 
       //! Retrieve label control for form.
       //! Usually the label control is only retrieved for alignment purposes.
      CTRL_LABEL& GetLabel (
			) { return (m_label); }
		  
		 //! Retrieve label control for form.
       //! Usually the slider control is only retrieved for alignment purposes.
		CTRL_SLIDER& GetSlider (
			) { return (m_slider); }
		
		//! Get current slider value, note it only has accuracy equal to the number of decimal places specified
		double GetValue (
			) const { return (m_slider.GetValue()); }
		
		//! Sets the color of the bar tool on the slider. Only call after create
		void SetColorBar (
			const COLOR &barcolor
			) { m_slider.SetColorBar(barcolor); }

		//!  Sets the color of the trough on the slider. Only call after create
		void SetColorTrough (
			const COLOR &troughcolor
			) { m_slider.SetColorTrough(troughcolor); }
		
		//! Set delegate called after value gets changed either via dragging and letting go of slider or via typing into control and enter/exiting
		void SetDelegateOnValueChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnValueChanged = delegate; }

		//! Called while user drags slider each time is moved, useful for dynamically updating
		void SetDelegateOnUserSliding (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnUserSliding = delegate; }

		//! Set Label
		void SetLabel(
			const MISTRING& label
			);

		//! Set number of decimal places to display.
		void SetPlaces (
			int places
			) { m_editctrl.SetFormat(MGUI::CTRL_EDIT_NUMBER::FORMAT_Decimal, places); }

			//! Set new range for the slider
		void SetRange (
			double min,										//!< New minimum
			double max									   //!< New maximum
			);

		//! Set step for rounding.
		//! If non-zero, values will be rounded to integral multiple of 'step' from minimum.
		//! @return true if results in value change, false if not.
		bool SetStep (
			double step,									//!< Step for rounding
			bool notify = true							//!< If true call OnValueChanged delegate if actual value changed
			);
		
		//! Set value for the slider
		void SetValue (
			double value,									//!< Value to set
			bool notify = true							//!< If true call OnValueChanged delegate if actual value changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SLIDER THISCLASS;

		DELEGATE_VOID_NOPARMS m_DelegateOnUserSliding;
		DELEGATE_VOID_NOPARMS m_DelegateOnValueChanged;
			
		CTRL_EDIT_NUMBER m_editctrl;
		CTRL_LABEL m_label;
		CTRL_SLIDER m_slider;

		void OnEditValueChanged ();
		void OnSliderDrag ();
		void OnSliderValueChanged ();

		#endif // GENERATING_DOXYGEN_OUTPUT	

	};

//=====================================================================================================================

}	// End namespace MGUI

#endif
