/**
 * \file mgui/timer.h
 * \brief Definitions for MGUI::TIMER class.
 *
 * \if NODOC
 * $Id: timer.h_v 1.3 2006/09/20 17:52:57 mju Exp $
 *
 * $Log: timer.h_v $
 * Revision 1.3  2006/09/20 17:52:57  mju
 * Don't inline x-specific stuff.
 *
 * Revision 1.2  2005/06/22 17:17:36  mju
 * Unify naming of timer ID to simplify code.
 * Add isActive method.
 *
 * Revision 1.1  2005/06/22 15:42:35  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TIMER_H
#define	INC_MGUI_TIMER_H

#ifndef	INC_MGUI_BASE_H
#include <mgui/base.h>
#endif

#ifndef	INC_MI32_XBASE_H
#include <mi32/xbase.h>
#endif

namespace MGUI {

//! Countdown timer, used to invoke delegate function when time expires.
//! Timers are only processed during normal event loop handling and thus
//! will not 'interrupt' program execution.  The specified time will be the
//! 'minimum' delay after which the delegate function will be called.
//! The actual time may be far longer if the program does significant
//! processing outside of event loops.
//! By default, timer delegates are only invoked during 'main loop' processing
//! to avoid undesired calls out of modal dialog loops.  To allow timer calls
//! for all event loops, use SetLimitToMainLoop(false).  In addition, if
//! a timer expires in a modal event loop and the auto-repeat option is not
//! used, the delegate will never be called.
class TIMER {
	public:

		// Constructor.
		TIMER ();

		// Destructor.
		~TIMER ();

		//! Activate the timer.
		//! If timer is already active, the countdown will be restarted using the new value.
		//! The number of active timers is limited on some platforms, an error will be returned
		//! if the maximum number of timers is already active.
		ERRVALUE Activate (
			double seconds,								//!< Countdown time in seconds
			bool autorepeat = false						//!< True to auto-repeat countdown after delegate invoked
			);

		//! Deactivate timer.
		void Deactivate (
			);

		//! Determine if timer is currently active.
		bool IsActive (
			) const { return (m_IdTimer != 0); }

		//! Set delegate to invoke when time expires.
		void SetDelegate (
			DELEGATE_VOID_NOPARMS delegate	//!< Delegate to set, use MakeDelegate() if non-static class method
			) { m_delegate = delegate; }

		//! Set whether to limit timer calls to main loop or allow in modal event loops.
		//! If not used, default is to limit to main loop.
		void SetLimitToMainLoop (
			bool LimitToMainLoop							//!< True to limit timer calls to main event loop
			) { m_LimitToMainLoop = LimitToMainLoop; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DELEGATE_VOID_NOPARMS m_delegate;

		#ifdef X_NATIVE
		XtIntervalId m_IdTimer;

		static void OnTimeX (TIMER*, XtIntervalId*);

		#else	// Windows
		UINT_PTR m_IdTimer;

		static VOID CALLBACK OnTimeW (HWND, UINT, UINT_PTR, DWORD);

		#endif

		double m_value;
		bool m_AutoRepeat;
		bool m_LimitToMainLoop;

		TIMER (const TIMER&);
		TIMER& operator= (const TIMER&);
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_TIMER_H
