/**
 * \file colrconv.h <mi32/colrconv.h>
 * \brief Definitions for color conversion functions
 *
 * \if NODOC
 * $Id: colrconv.h_v 1.44 2008/10/15 22:07:29 scowan Exp $
 *
 * $Log: colrconv.h_v $
 * Revision 1.44  2008/10/15 22:07:29  scowan
 * Changed color blend defines to not conflict with raster layer enum.
 *
 * Revision 1.43  2007/04/17 13:20:03  mju
 * Remove unused prototype.
 *
 * Revision 1.42  2007/03/30 17:46:31  mju
 * Use contrastparm.
 * Remove sample/zoom parms from start as no longer supported.
 *
 * Revision 1.40  2007/03/05 16:45:06  mju
 * No dft for flags in deprecated method.
 *
 * Revision 1.39  2007/01/22 14:15:00  mju
 * Remove colmapinfo variants.
 *
 * Revision 1.38  2007/01/22 14:06:15  mju
 * Deprecations for colmapinfo variants.
 *
 * Revision 1.37  2007/01/19 16:09:26  mju
 * Use color_array.
 *
 * Revision 1.36  2006/11/29 18:00:33  scowan
 * Removed unused function prototypes.
 *
 * Revision 1.35  2006/11/20 20:54:09  scowan
 * Do not deprecate the contparm versions until raster dispparm uses the new contrast class.
 *
 * Revision 1.34  2006/11/20 18:31:22  mju
 * Don't have default for flags where ambiguous.
 *
 * Revision 1.33  2006/11/20 18:25:29  mju
 * Add setSrcSingle variants using contrastparm and deprecate others.
 *
 * Revision 1.32  2006/11/20 16:11:24  mju
 * Remove optmparm.
 *
 * Revision 1.31  2006/10/27 21:16:26  mju
 * contparm now class.
 *
 * Revision 1.30  2006/10/18 15:34:58  mju
 * Deprecate colorcomphandle typedef.
 * Remove unnecessary rvc hdrs.
 *
 * Revision 1.29  2005/09/08 13:52:53  dwilliss
 * The new SetSrcSingle that takes colorspace should take the colormap as a
 * ponter to simple array of colors, not old colmapinfo
 *
 * Revision 1.28  2005/09/07 23:01:45  dwilliss
 * Added SetSrcSingle version that takes a COLORSPACE
 *
 * Revision 1.27  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.26  2002/10/17 19:06:03  mju
 * Add SetDestSingle using COLORSPACE.
 *
 * Revision 1.25  2002/10/16 22:03:46  mju
 * Make COLORCOMPDEST enum and use in SetDestSingle.
 *
 * Revision 1.24  2002/09/06 13:33:04  mju
 * Remove old convcomp fns.
 *
 * Revision 1.23  2002/08/15 15:49:50  dwilliss
 * Never mind...
 *
 * Revision 1.22  2002/08/15 15:48:47  dwilliss
 * Prototype for new SetDestSingle method that takes an MGD::DEVICE& instead
 * of all the device parameters
 *
 * Revision 1.21  2002/01/02 15:20:23  mju
 * Change COLORCOMPSRCPARM to use COLOR array.
 *
 * Revision 1.20  2001/12/28 23:14:40  dwilliss
 * Allow the overloaded functions now
 *
 * Revision 1.19  2001/12/27 17:43:41  dwilliss
 * Ifdef out the simple-array versions for now.  Causes ambiguious function
 * calls if you pass NULL for the color map and Mike's in the middle of
 * a rewrite of GRE so he can't check in the fix to make it unambiguous
 *
 * Revision 1.18  2001/12/27 15:09:09  dwilliss
 * Added SetSrc/DestSingle methods that don't take any form of colormap
 * to deal with ambiguous call if you try to pass NULL
 *
 * Revision 1.17  2001/12/27 14:18:55  mju
 * Temporarily remove new overloads to resolve ambiguity.
 *
 * Revision 1.16  2001/12/26 23:42:47  dwilliss
 * Previous revision, I forgot to save changes from the editor first.
 * Prototypes for SetSrc/DestSingle functions that take COLOR_ARRAY.
 *
 * Revision 1.14  2001/09/10 14:17:52  vdronov
 * added flag MaskToAlpha
 *
 * Revision 1.13  2001/05/10 16:25:59  mju
 * Deprecate old convcomp functions.
 *
 * Revision 1.1  1997/04/30  17:05:37  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_COLRCONV_H
#define	INC_MI32_COLRCONV_H

#ifndef	INC_MI32_COLORSPC_H
	#include <mi32/colorspc.h>
#endif

#ifndef	INC_MI32_NUMTYPE_H
	#include <mi32/numtype.h>
#endif

#ifndef	INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class COLOR_ARRAY;
struct CBPARMS;
struct DISPLAYINFO;
class CONTRASTPARM;
namespace MGD {
	class DEVICE;
	}
#endif

/*----------------------------------------------------------------------------*/
/*		Definitions for color-composite conversion										*/
/*----------------------------------------------------------------------------*/

enum COLORCOMPFLAGS {
	COLORCOMPFLAG_None =				0,
	COLORCOMPFLAG_DoContrast =		0x00000001,		//!< Perform contrasting, for single-raster input only
	COLORCOMPFLAG_UseColMap =		0x00000002,		//!< Use colormap, for single raster input only
	COLORCOMPFLAG_Out32For24 =		0x00000004,		//!< Output 32-bit for 24-bit depth
	COLORCOMPFLAG_ScaleInt255 =	0x00000008,		//!< Input intensity (or brightness) ranges from 0-255 instead of 0-100
	COLORCOMPFLAG_ScaleSat255 =	0x00000010,		//!< Input saturation ranges from 0-255 instead of 0-100
	COLORCOMPFLAG_ScaleHue255 =	0x00000020,		//!< Input hue ranges from 0-255 instead of 0-360
	COLORCOMPFLAG_AdjustInt4 =		0x00000040,		//!< Adjust intensity using 4th band rather than replacing with 4th band
	COLORCOMPFLAG_OutPixel =		0x00000080,		//!< Output Pixel buffer
	COLORCOMPFLAG_DoBlendMask =	0x00000100,		//!< Use blending masks (ColorComp... functions only)
	COLORCOMPFLAG_MaskToAlpha =	0x00000200,		//!< Set alpha bits if mask is true, clear it otherwise
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(COLORCOMPFLAGS)
#endif

#define	CONVCOMP_DoContrast COLORCOMPFLAG_DoContrast
#define	CONVCOMP_UseColMap COLORCOMPFLAG_UseColMap
#define	CONVCOMP_Out32For24 COLORCOMPFLAG_Out32For24
#define	CONVCOMP_ScaleInt255 COLORCOMPFLAG_ScaleInt255
#define	CONVCOMP_ScaleSat255 COLORCOMPFLAG_ScaleSat255
#define	CONVCOMP_ScaleHue255 COLORCOMPFLAG_ScaleHue255
#define	CONVCOMP_AdjustInt4 COLORCOMPFLAG_AdjustInt4
#define	CONVCOMP_OutPixel COLORCOMPFLAG_OutPixel
#define	CONVCOMP_DoBlendMask COLORCOMPFLAG_DoBlendMask
#define	CONVCOMP_MaskToAlpha COLORCOMPFLAG_MaskToAlpha


//! RGBI - Intensity conversion models for RASTDISPMODE_MultiRGBI
//! Changed to not conflict with GRE::LAYER_RASTER::COLORBLEND enum
#define	COLOR_BLEND_RGBI_HIS			1
#define	COLOR_BLEND_RGBI_HBS			2
#define	COLOR_BLEND_RGBI_Ratio		3
#define	COLOR_BLEND_RGBI_Brovey		4
#define	COLOR_BLEND_RGBI_LAST		4

//! Types for ColorCompSetDestSingle
enum COLORCOMPDEST {
	COLORCOMPDEST_ColorMapped =	1,		//!< 8- or 16-bit color-mapped
	COLORCOMPDEST_RGB =				2,		//!< 16- or 24-bit RGB/BGR packed
	COLORCOMPDEST_Pixel =			3,		//!< Array of Pixels
	COLORCOMPDEST_Image =			4		//!< Suitable for MgdPutImageSpan or XImage ZPixmap
	};


/*----------------------------------------------------------------------------*/
/*		Color conversion parameters structures												*/
/*----------------------------------------------------------------------------*/

struct COLORCOMPSRCPARM {
	UINT8 CompType;									//!< RASTDISPMODE_...
	UINT8 BlendMode;									//!< COLORBLEND_RGBI_...
	UINT8 BlendFactor;								//!< Blending factor 1-254 for COLORBLEND_RGBI_Ratio
	UINT32 flags;										//!< CONVCOMP_... flags (defined above)
	const CONTRASTPARM *PtrsContrastParm[4];	//!< Contrast parameters for each component
	UINT32 CellType[4];								//!< RVC::IMAGE::CELLTYPE for each component	
	int NumColors;
	const COLOR* colmap;
	};

/*----------------------------------------------------------------------------*/
/*		Color conversion "handle", should not be directly referenced				*/
/*----------------------------------------------------------------------------*/

struct COLORCOMPSTRUCT;


/*----------------------------------------------------------------------------*/
/*		Function prototypes																		*/
/*----------------------------------------------------------------------------*/

//! Convert Multiple rasters to Single raster.
int ColorCompConvMtoS (
	COLORCOMPSTRUCT *ccp, 
	void *odata, 
	UINT8 *omask,								//!<  Blending mask returned 
	const void *idata1,
	const void *idata2,
	const void *idata3,
	const void *idata4, 
	const UINT8 *imask, 						//!<  Input blending mask 
	INT32 inpcols
	);

//! Convert Single composite color line to Multiple R-G-B line.
int ColorCompConvStoRGB (
	COLORCOMPSTRUCT *ccp, 
	UINT8 *ored, 
	UINT8 *ogrn, 
	UINT8 *oblu, 
	UINT8 *omask, 								//!<  Blending mask returned 
	const void *idata, 
	const UINT8 *imask, 						//!<  Input blending mask 
	INT32 inpcols
	);

//! Convert single input raster to single output raster.
int ColorCompConvStoS (
	COLORCOMPSTRUCT *ccp, 
	void *odata, 
	UINT8 *omask, 								//!<  Blending mask returned 
	const void *idata, 
	const UINT8 *imask, 						//!<  Input blending mask 
	INT32	inpcols
	);

//! Create color-composite conversion handle.
int ColorCompCreate (
	COLORCOMPSTRUCT **rethandle
	);

//! Destroy color-composite conversion handle.
void ColorCompDestroy (
	COLORCOMPSTRUCT *ccp
	);

//! Set color balance parameters.
int ColorCompSetColorBalance (
	COLORCOMPSTRUCT *ccp, 
	CBPARMS *cbparm
	);

//! Set parameters for separate RGB output.
int ColorCompSetDestRGB (
	COLORCOMPSTRUCT *ccp, 
	int type = 0,
	UINT32 flags = 0
	);

//! Set parameters for single-raster output.
int ColorCompSetDestSingle (
	COLORCOMPSTRUCT *ccp, 
	COLORCOMPDEST desttype, 
	const DISPLAYINFO *pDisplayInfo, 
	const COLOR_ARRAY *pColorMap = 0,
	const void *notused = 0,
	UINT32 flags = 0
	);

//! Set parameters for single-raster output using COLORSPACE.
int ColorCompSetDestSingle (
	COLORCOMPSTRUCT *ccp, 
	COLORSPACE ColorSpace,								//!< Destination COLORSPACE
	const COLOR_ARRAY *colormap = 0,					//!< Color palette for 'Indexed' colorspaces
	UINT32 flags = 0										//!< CONVCOMP flags
	);

//! Set parameters for single-raster output.
int ColorCompSetDestSingle (
	COLORCOMPSTRUCT *ccp, 
	MGD::DEVICE& dev
	);

//! Set parameters for single-raster output.
inline int ColorCompSetDestSingle (
	COLORCOMPSTRUCT *ccp, 
	COLORCOMPDEST desttype,
	const DISPLAYINFO *pDisplayInfo, 
	UINT32 flags
	) {
	return (ColorCompSetDestSingle(ccp, desttype, pDisplayInfo, static_cast<COLOR_ARRAY*>(0), 0, flags));
	}

//! Set source parameters.
int ColorCompSetSrcParm (
	COLORCOMPSTRUCT *ccp, 
	COLORCOMPSRCPARM *parm
	);

//! Set parameters for single-raster input using COLORSPACE.
ERRVALUE ColorCompSetSrcSingle (
	COLORCOMPSTRUCT *ccp,
	COLORSPACE ColorSpace,							//!< Source colorspace
	const COLOR_ARRAY *pColorMap,					//!< Color palette, 0 if unused
	const CONTRASTPARM *pContrastParm,			//!< Contrast parameters, 0 if unused
	COLORCOMPFLAGS flags = COLORCOMPFLAG_None
	);

//! Set parameters for single-raster input using RVC::IMAGE::CELLTYPE.
ERRVALUE ColorCompSetSrcSingle (
	COLORCOMPSTRUCT *ccp,
	RVC::IMAGE::CELLTYPE CellType,
	const COLOR_ARRAY *pColorMap,					//!< Color palette, 0 if unused
	const CONTRASTPARM *pContrastParm,			//!< Contrast parameters, 0 if unused
	COLORCOMPFLAGS flags = COLORCOMPFLAG_None
	);

//! Set parameters for single-raster input using NUMTYPE.
ERRVALUE ColorCompSetSrcSingle (
	COLORCOMPSTRUCT *ccp,
	NUMTYPE NumType,
	const COLOR_ARRAY *pColorMap,					//!< Color palette, 0 if unused
	const CONTRASTPARM *pContrastParm,			//!< Contrast parameters, 0 if unused
	COLORCOMPFLAGS flags = COLORCOMPFLAG_None
	);

//!	Initialize and allocate memory for selected conversion type(s).
int ColorCompStart (
	COLORCOMPSTRUCT *ccp, 
	INT32 maxcols
	);

//! Stop color conversion without destroying handle.
void ColorCompStop (
	COLORCOMPSTRUCT *ccp
	);


#endif	// INC_MI32_COLRCONV_H
