/**
 * \file critsec.h <mi32/critsec.h>
 * \brief Definition for Windows Critical section objects
 *
 * \if NODOC
 * $Id: critsec.h_v 1.19 2008/06/11 21:08:51 mju Exp $
 *
 * $Log: critsec.h_v $
 * Revision 1.19  2008/06/11 21:08:51  mju
 * Add lock ctor using critsec ptr.
 *
 * Revision 1.18  2008/04/11 15:37:29  scowan
 * More doc updates.
 *
 * Revision 1.17  2008/03/20 16:37:10  scowan
 * Updated docs on spin count.
 *
 * Revision 1.16  2007/09/18 20:51:27  scowan
 * Removed deprecated typedef.
 *
 * Revision 1.15  2007/08/30 22:53:48  scowan
 * Added bool to lock class to properly support release method.
 *
 * Revision 1.14  2007/08/17 17:48:49  scowan
 * Added release method on lock class.
 *
 * Revision 1.13  2007/04/18 20:41:45  mju
 * Move lock defn inside criticalsection so hopefully doxygen will handle correctly.
 *
 * Revision 1.12  2007/04/02 15:41:05  mju
 * Try again to fix doxygen by excluding doc of typedef.
 *
 * Revision 1.11  2007/03/30 19:53:12  mju
 * Attempt to fix doxygen.
 *
 * Revision 1.10  2007/02/15 16:50:26  scowan
 * Fixed invalid comments.
 *
 * Revision 1.9  2007/02/02 23:38:42  scowan
 * Moved previous standalone thread classes into mithread namespace.
 *
 * Revision 1.8  2007/01/25 16:15:28  scowan
 * Maybe fixed for unix.
 *
 * Revision 1.7  2007/01/25 15:59:42  scowan
 * Updated for unix.
 *
 * Revision 1.6  2005/07/12 20:28:14  dwilliss
 * Added an Enter method
 *
 * Revision 1.5  2005/05/16 16:19:38  scowan
 * Move lock class into dll and export.
 *
 * Revision 1.4  2004/10/07 15:35:32  mju
 * Add copy ctor and op=.
 *
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2002/01/16 18:13:32  scowan
 * Do stupid thing for MAC.
 *
 * Revision 1.1  2001/12/06 17:46:22  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_CRITSEC_H
#define	INC_MI32_CRITSEC_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace MITHREAD {

//! Critical section class for Windows and UNIX platforms.
class CLASSLIBEXPORT CRITICALSECTION {
	public:

		//! Control entering an instance of a CRITICALSECTION upon initialization and
		//! released when out of scope.
		class CLASSLIBEXPORT LOCK {
			public:
				//! Default constructor - used to implement TryEnter() paradigm.
				LOCK ();

				//! Constructor - Get critical section or wait until it is available
				explicit LOCK (
					CRITICALSECTION& CritSec,
					bool EnterSection = true			//!< Flag to allow setting of non-multithread mode
					);

				//! Constructor - Get critical section or wait until it is available
				explicit LOCK (
					CRITICALSECTION *pCritSec			//!< Critical section pointer, if NULL no lock will be obtained
					);

				//! Destructor - Releases critical section				
				~LOCK ();

				//! Enter a critical section.
				void Enter (
					CRITICALSECTION& CritSec
					);

				//! Release the critical section lock.
				void Release (
					);

				//! Check if it is possible to enter a critical section.
				//! Returns immediately, regardless of 'spin count' setting.
				//! @return 'True' if entered critical section, 'false' if not
				bool TryEnter (
					CRITICALSECTION& CritSec
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				CRITICALSECTION *m_pCritSec;
				bool XXX_Obsolete;
				LOCK (const LOCK&);
				LOCK& operator= (const LOCK&);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Default constructor.
		//! A "critical section" is a fancy name for an in-process only mutex.  Given that, the "SpinCount" is used to delay 
		//! the acquisition of that system mutex and the expensive transfer into and out of kernel mode.  The use of the 
		//! "SpinCount" is to determine how many times, in a loop, that a check is done to see if the lock is granted.
		//! The downside to this is that the spinning takes up CPU time in the loop doing the checks.  Therefore using the 
		//! "SpinCount" is a gamble to see if the loop method will take less time than the kernel transitions for the mutex object.
		//! Rule of thumb: do not use the "SpinCount" to protect code that itself makes a kernel transition (I/O calls) or is 
		//! a time consuming operation.
		//! From MSDN Docs: "You can improve performance significantly by choosing a small spin count for a critical section of 
		//! short duration. The heap manager uses a spin count of roughly 4000 for its per-heap critical sections. This gives great 
		//! performance and scalability in almost all worst-case scenarios."
		CRITICALSECTION (
			unsigned long SpinCount = 0	//!< How many times to check and yield before allocating a system resource
			);

		//! Destructor.
		~CRITICALSECTION (
			);

		//! Set spin count for the critical section
		//! See the constructor for the documentation on how to use the "SpinCount".
		unsigned long SetSpinCount (
			unsigned long SpinCount			//!< How many times to check and yield before allocating a system resource
			);

	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_pPriv;

		CRITICALSECTION (const CRITICALSECTION& rhs);
		CRITICALSECTION& operator= (const CRITICALSECTION& rhs);

		friend class LOCK;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of MITHREAD namespace

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_CRITSEC_H

