/**
*	\file <mi32/fixedstrchar.h>
*	\brief FIXEDSTRCHAR<> template definition
*
*	\if NODOC
*	$Id: fixedstrchar.h_v 1.2 2007/05/07 13:32:07 mju Exp $
*
*	$Log: fixedstrchar.h_v $
*	Revision 1.2  2007/05/07 13:32:07  mju
*	Remove unneeded hdrs.
*
*	Revision 1.1  2007/05/07 13:30:07  mju
*	Initial revision
*
*	\endif
**/

#ifndef	INC_MI32_FIXEDSTRCHAR_H
#define	INC_MI32_FIXEDSTRCHAR_H

#ifndef INC_MI32_INIBASE_H
	#include <mi32/inibase.h>
#endif

#ifndef INC_MEMORY_H
	#include <memory.h>		// For memcpy
	#define INC_MEMORY_H
#endif

//! Fixed-size array-based 'char' string.
//!	This class is designed to replace fixed size 'char' array string constructs in other classes and structures.
//! It is not meant as a STRING class implementation.  Therefore, the class cannot have virtual methods nor any
//!	other members except for the string storage itself.
template <int _CT> class FIXEDSTRCHAR {
	public:

		//! Default constructor, clears the string.
		FIXEDSTRCHAR (									
			) { Clear(); }

		//! Copy constructor.
		FIXEDSTRCHAR (									
			const FIXEDSTRCHAR& rhs
			) { memcpy(m_String, rhs.m_String, sizeof(m_String)); }

		//! Construct from char string.
		FIXEDSTRCHAR (									
			const char* string
			) { Clear(); strncpy(m_String, string, _CT - 1); }

		//! Destructor.
		~FIXEDSTRCHAR (									
			) { m_String[0] = 0; }

		//! Assignment operator.
		FIXEDSTRCHAR& operator= (						
			const FIXEDSTRCHAR& rhs				//!< String to assign from
			) {
			if (this != &rhs) {
				memcpy(m_String, rhs.m_String, sizeof(m_String));
				}
			return (*this);
			}

		//! Assign from char string.
		FIXEDSTRCHAR& operator= (					
			const char* rhs					//!< String to assign from
			) {
			if (m_String != rhs) {
				Clear();
				strncpy(m_String, rhs, _CT - 1);
				}
			return (*this);
			}

		//! Concatenate operator.
		FIXEDSTRCHAR& operator+= (						
			const FIXEDSTRCHAR& rhs				//!< String to concatinate from
			) {
			strncat(m_String, rhs.m_String, _CT - (strlen(m_String) + 1));
			return (*this);
			}

		//! Concatenate from char string.
		FIXEDSTRCHAR& operator+= (						
			const char* rhs					//!< String to concatinate from
			) {
			strncat(m_String, rhs, _CT - (strlen(m_String) + 1));
			return (*this);
			}
			
		//! Equality.
		bool operator== (
			const FIXEDSTRCHAR& rhs
			) const { return (Compare(rhs) == 0); }

		//! Equality.
		bool operator== (
			const char *rhs
			) const { return (Compare(rhs) == 0); }

		//! Inequality.
		bool operator!= (
			const FIXEDSTRCHAR& rhs
			) const { return (!(*this == rhs)); }

		//! Inequality.
		bool operator!= (
			const char *rhs
			) const { return (!(*this == rhs)); }

		//! Less Than
		bool operator< (
			const FIXEDSTRCHAR& rhs
			) const { return (Compare(rhs) < 0); }

		//! Less Than
		bool operator< (
			const char *rhs
			) const { return (Compare(rhs) < 0); }

		//! Less Than or Equal to
		bool operator<= (
			const FIXEDSTRCHAR& rhs
			) const { return (Compare(rhs) <= 0); }

		//! Less Than or Equal to
		bool operator<= (
			const char *rhs
			) const { return (Compare(rhs) <= 0); }

		//! Greater Than
		bool operator> (
			const FIXEDSTRCHAR& rhs
			) const { return (Compare(rhs) > 0); }

		//! Greater Than
		bool operator> (
			const char *rhs
			) const { return (Compare(rhs) > 0); }

		//! Greater Than or Equal to
		bool operator>= (
			const FIXEDSTRCHAR& rhs
			) const { return (Compare(rhs) >= 0); }

		//! Greater Than or Equal to
		bool operator>= (
			const char *rhs
			) const { return (Compare(rhs) >= 0); }

		//! Cast to <const char*>.
		operator const char* (
			) const { return (m_String); }

		//! Assign from character string.
		void Assign (
			const char* name						//!< Character string to assign from
			) { strncpy(m_String, name, _CT - 1); }

		//! Assign from char string with a max length given
		void Assign (								 	
			const char* name,						//!< char string to assign from
			int len
			) { strncpy(m_String, name, MIN(_CT - 1, len)); }

		//! Clear the whole string.
		void Clear (									
			) { memset(m_String, 0, sizeof(m_String)); }
		
		//! Case-sensitive comparison, similar to strcmp.
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int Compare (
			const FIXEDSTRCHAR& str						//!< String to compare to
			) const { return (strcmp(m_String, str.m_String)); }

		//! Case-sensitive comparison, (similar to strcmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int Compare (
			const char *str						//!< String to compare to
			) const { return (strcmp(m_String, str)); }

		//! Case-insensitive comparison (similar to stricmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int CompareNoCase (
			const FIXEDSTRCHAR& str						//!< String to compare to
			) const { return (stricmp(m_String, str.m_String)); }

		//! Case-insensitive comparison (similar to stricmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int CompareNoCase (
			const char *str						//!< String to compare to
			) const { return (stricmp(m_String, str)); }
			
		//! Find out if the string is empty.	
		bool IsEmpty (									
			) const { return (m_String[0] == 0); }
			
		//! Set the string according to values in ini file
		//! @return: 'true' if entry found and information read, false if empty field or field does not exist
		bool IniRead (
			INIHANDLE inih,
			const char *group,
			const char*const field
			) { return (_iniReadString(inih, group, field, INITYPE_ASCII, m_String, _CT * sizeof(char)) > 0); }

		//! Save the string to values in ini file
		void IniWrite (
			INIHANDLE inih,
			const char *group,
			const char*const field
			) { _iniWriteString(inih, group, field, INITYPE_ASCII, m_String); }

		//! Return length of string, not including terminator.
		int GetLength (						  		
			) const { return (strlen(m_String)); }

		//! Return maximum string size, not including terminator.
		int GetMaxSize (						  		
			) const { return (_CT-1); }

		//! Get tail of string having up to specified number of characters.
		const char* GetTail (
			int maxlen
			) const { int len = strlen(m_String); return ((len <= maxlen) ? m_String : (m_String + (len - maxlen))); }
			
		//! Pad the string with a char to the end of the buffer
		void PadToEnd (
			char item
			) {
			for (int j = strlen(m_String);(j < _CT-1);++j) m_String[j] = item;
			m_String[_CT-1] = 0;
			}
			
		//! Set the string to lower case letters
		void SetLowerCase (
			) { strlwr(m_String); }

		//! Set the string to upper case letters
		void SetUpperCase (
			) { strupr(m_String); }
			
		//! Force termination of string at the end.
		void Terminate (								
			) { m_String[_CT - 1] = 0; }

		//! Truncate the string, has no effect if the string is shorter than NewLength.
		void Truncate (									
			int NewLength
			) { if (NewLength >= 0 && NewLength < (_CT - 1)) m_String[NewLength] = 0; }

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		char m_String[_CT];
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_FIXEDSTRCHAR_H

