/**
 * \file mi32/grid2d.h
 * \brief GRID2D definitions
 *
 * \if NODOC
 * $Id: grid2d.h_v 1.4 2012/09/28 19:41:06 mju Exp $
 *
 * $Log: grid2d.h_v $
 * Revision 1.4  2012/09/28 19:41:06  mju
 * Add setInterval with single value.
 *
 * Revision 1.3  2012/09/17 13:30:17  mju
 * Attempt to fix dox.
 *
 * Revision 1.2  2012/09/13 15:44:51  mju
 * Add calcIdxRange.
 *
 * Revision 1.1  2012/08/31 21:12:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_GRID2D_H
#define	INC_MI32_GRID2D_H

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

//=====================================================================================================================

//! Bounded 2D grid definition.
//! The range of values X for cell C is defined as C <= X < C+1 (lower bound is closed, upper bound is open).
class GRID2D {
	public:

		//! Default constructor.
		GRID2D (
			): m_Origin(0,0), m_Interval(0,0), m_IntervalInv(0,0), m_Size(0,0) { }

		//! Construct with specified origin and interval.
		GRID2D (
			const DPOINT2D& origin,		//!< Grid origin
			const DPOINT2D& interval	//!< Grid interval
			): m_Origin(origin), m_Interval(interval), m_Size(0,0) {
			m_IntervalInv.x = (m_Interval.x != 0) ? (1.0 / m_Interval.x) : 0;
			m_IntervalInv.y = (m_Interval.y != 0) ? (1.0 / m_Interval.y) : 0;
			}

		//! Calculate X coordinate from index.
		double CalcCoordX (
			INT32 IdxX						//!< Grid X index
			) const { return (IdxX * m_Interval.x + m_Origin.x); }

		//! Calculate Y coordinate from index.
		double CalcCoordY (
			INT32 IdxY						//!< Grid Y index
			) const { return (IdxY * m_Interval.y + m_Origin.y); }

		//! Calculate linear array index from X/Y indices.
		INT32 CalcIdx (
			INT32 IdxX,					//!< X index (array column)
			INT32 IdxY					//!< Y index (array row)
			) const { return (IdxY * m_Size.x + IdxX); }

		//! Calculate linear array index from point coordinates.
		INT32 CalcIdx (
			const DPOINT2D& pt		//!< Point coordinates
			) const { return (CalcIdx(CalcIdxX(pt),CalcIdxY(pt))); }

		//! Calculate range of index values from source rectangle.
		void CalcIdxRange (
			const DRECT2D& srcrect,	//! Source rectangle
			LRECT2D& idxrect			//! Index rectangle returned
			) const { idxrect.Set(CalcIdxX(srcrect.xinit),CalcIdxY(srcrect.yinit),CalcIdxX(srcrect.xlast),CalcIdxY(srcrect.ylast)); }

		//! Calculate linear array index from point coordinates.
		//! @return -1 for point out of range.
		INT32 CalcIdxValidated (
			double x,
			double y
			) const {
			double PosX = CalcPosX(x);
			if (PosX < 0 || PosX >= m_Size.x) return (-1);
			double PosY = CalcPosY(y);
			if (PosY < 0 || PosY >= m_Size.y) return (-1);
			return (CalcIdx(FAST_FLOOR(PosX),FAST_FLOOR(PosY)));
			}

		//! Calculate linear array index from point coordinates.
		//! @return -1 for point out of range.
		INT32 CalcIdxValidated (
			const DPOINT2D& pt		//!< Point coordinates
			) const { return (CalcIdxValidated(pt.x,pt.y)); }

		//! Calculate X index (grid column).
		INT32 CalcIdxX (
			double x							//!< X coordinate
			) const { return (FAST_FLOOR(CalcPosX(x))); }

		//! Calculate X index (grid column).
		INT32 CalcIdxX (
			const DPOINT2D& pt			//!< Point coordinates
			) const { return (CalcIdxX(pt.x)); }

		//! Calculate Y index (grid row).
		INT32 CalcIdxY (
			double y							//!< Y coordinate
			) const { return (FAST_FLOOR(CalcPosY(y))); }

		//! Calculate Y index (grid row).
		INT32 CalcIdxY (
			const DPOINT2D& pt			//!< Point coordinates
			) const { return (CalcIdxY(pt.y)); }

		//! Calculate X position from X coordinate.
		double CalcPosX (
			double x							//!< X coordinate
			) const { return ((x - m_Origin.x) * m_IntervalInv.x); }

		//! Calculate Y position.
		double CalcPosY (
			double y							//!< Y coordinate
			) const { return ((y - m_Origin.y) * m_IntervalInv.y); }

		//! Get interval interval.
		const DPOINT2D& GetInterval () const
			{ return (m_Interval); }

		//! Determine if grid specification contains point.
		bool Contains (
			const DPOINT2D& pt		//!< Point coordinates
			) const { return (pt.x >= m_Origin.x && pt.y >= m_Origin.y && pt.x < m_Origin.x + m_Interval.x * m_Size.x && pt.y < m_Origin.y + m_Interval.y * m_Size.y); }

		//! Get size of array.
		UINT32 GetArraySize () const
			{ return (m_Size.x * m_Size.y); }

		//! Get grid origin.
		const DPOINT2D& GetOrigin () const
			{ return (m_Origin); }

		//! Get size (grid dimensions).
		const LPOINT2D& GetSize () const
			{ return (m_Size); }

		//! Set grid interval.
		void SetInterval (
			const DPOINT2D& interval	//!< Grid interval
			) {
			m_Interval = interval;
			m_IntervalInv.x = (m_Interval.x != 0) ? (1.0 / m_Interval.x) : 0;
			m_IntervalInv.y = (m_Interval.y != 0) ? (1.0 / m_Interval.y) : 0;
			}

		//! Set grid interval.
		void SetInterval (
			double interval				//!< Grid interval
			) {
			m_Interval.x = m_Interval.y = interval;
			m_IntervalInv.x = m_IntervalInv.y = (interval != 0) ? (1.0 / interval) : 0;
			}

		//! Set grid origin.
		void SetOrigin (
			const DPOINT2D& origin		//!< Grid origin
			) { m_Origin = origin; }

		//! Set grid size.
		void SetSize (
			const LPOINT2D& size			//!< Size in grid cells
			) { m_Size = size; }

		//! Set grid size.
		void SetSize (
			UINT32 SizeX,					//!< X size (grid cells)
			UINT32 SizeY					//!< Y size (grid cells)
			) { m_Size.x = SizeX; m_Size.y = SizeY; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DPOINT2D m_Origin;
		DPOINT2D m_Interval;
		DPOINT2D m_IntervalInv;
		LPOINT2D m_Size;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_GRID2D_H
