/******************************************************************************,
 *
 * \file mi32/mg2enums.h
 * \brief enums used by the MGD code
 *
 * \if NODOC
 * $Log: mg2enums.h_v $
 * Revision 1.15  2013/03/12 23:12:51  ldyas
 * added enum for simple border capability
 *
 * Revision 1.14  2011/12/21 22:29:47  mju
 * Remove unused pixelfunc enum values.
 *
 * Revision 1.13  2011/12/15 21:41:11  mju
 * Add capability_Stipple.
 *
 * Revision 1.12  2009/09/28 17:44:38  mju
 * Add direction enum.
 *
 * Revision 1.11  2007/12/24 16:25:48  scowan
 * Switched to using source and dest for alpha blend types.
 *
 * Revision 1.10  2007/12/21 23:25:21  scowan
 * Added two more opacity blend modes.
 *
 * Revision 1.9  2007/10/05 21:25:24  dwilliss
 * Added a capability mask for devices supporting BitmapFillRGBA
 *
 * Revision 1.8  2007/07/10 17:58:24  dwilliss
 * renamed the enum to opacity mode since that's what it really is
 *
 * Revision 1.7  2007/07/10 15:11:16  dwilliss
 * changed the names of an enum
 *
 * Revision 1.6  2007/07/10 14:32:25  dwilliss
 * added TRANSPARENCY_MODE
 *
 * Revision 1.5  2007/06/18 15:49:17  ldyas
 * comma missed.
 *
 * Revision 1.4  2007/06/18 15:42:47  ldyas
 * extended capabilities
 *
 * Revision 1.3  2007/01/25 21:10:52  dwilliss
 * Added COMPONENT enum
 *
 * Revision 1.2  2005/01/26 18:15:00  mju
 * Remove cap_scanColorUse.
 *
 * Revision 1.1  2004/05/25 19:26:41  dwilliss
 * Initial revision
 *
 * \endif
 */

#ifndef INC_MI32_MG2ENUMS_H
#define INC_MI32_MG2ENUMS_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif


namespace MGD {

//! Mode setting for how to deal with opacity when blending translucent 
//! colors together
enum OPACITY_MODE {
	OPACITY_MODE_MaxOpacity = 0,		//!< Resulting opacity is the maximum of the source and destination opacity
	OPACITY_MODE_Subtractive = 1,		//!< 50% transparent * 50% transparent = 25% transparent.
	OPACITY_MODE_Source = 2,			//!< Use the transparency specified in the color given, overrides the value in the destination.  Allows rendering in full transparency.
	OPACITY_MODE_Destination = 3		//!< Use the transparency specified in the destination, ignores the source value.
	};

enum FONTTYPE {
	FONTTYPE_OF = 0,
	FONTTYPE_TTF = 1
	};

enum ARROWTYPE {
	ARROWTYPE_FilledTriangle = 0,
	ARROWTYPE_UnfilledTriangle,
	ARROWTYPE_Open,
	ARROWTYPE_FilledArc
	};

enum DIRECTION {
	DIRECTION_None =		0x0,
	DIRECTION_Up =			0x1,
	DIRECTION_Down =		0x2,
	DIRECTION_Left =		0x4,
	DIRECTION_Right =		0x8,
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DIRECTION)
#endif

//! Pixel function for drawing.
enum PIXELFUNC {
	PIXELFUNC_Clear = 0,			//!< 0
	PIXELFUNC_Copy,				//!< src (default)
	PIXELFUNC_ExclusiveOr,		//!< src XOR dst
	PIXELFUNC_Invert,				//!< NOT dst
	PIXELFUNC_Set					//!< 1
	};

enum SHAPECLOSE {
	SHAPECLOSE_None = 0,	
	SHAPECLOSE_Direct,
	SHAPECLOSE_ViaCenter
	};

enum DEPTHTEST {
	DEPTHTEST_None	= 0,
	DEPTHTEST_SetOnly = 1,
	DEPTHTEST_CheckAndSet = 2
	};

enum LINECAP {
	LINECAP_NotLast = 0,
	LINECAP_Butt,
	LINECAP_Round,
	LINECAP_Projecting
	};

enum LINEJOIN {
	LINEJOIN_Miter = 0,
	LINEJOIN_Round,
	LINEJOIN_Bevel
	};

enum POLYSHAPE {
	POLYSHAPE_Complex = 0,		//!< Polygon may or may not be convex
	POLYSHAPE_NonConvex,			//!< Polygon is known to be non-convex
	POLYSHAPE_Convex				//!< Polygon is convex (doesn't double back on itself)
	};

enum CAPABILITY {
	CAPABILITY_None =					 			0,
	CAPABILITY_Transparency =					0x00000001,	//!< Can do transparency
	CAPABILITY_3D =								0x00000002,
	CAPABILITY_InterfaceText =		 			0x00000004,
	CAPABILITY_Stipple =							0x00000008,	//!< Stipple mask supported
	CAPABILITY_CopyRect =						0x00000010,
	CAPABILITY_GetImage =						0x00000020,
	CAPABILITY_MaskInImageSpan =				0x00000040,	//!< PutImageSpan can use a mask array 
	CAPABILITY_TransparencyIsExpensive =	0x00000080,	//!< Can do it, but don't bother for anti-aliasing text.
	CAPABILITY_ComplexClip = 					0x00000100,	//!< Device handles clip regions
	CAPABILITY_BitmapFill = 					0x00000200,	//!< Implements its own, optimized bitmap pattern fill in FillSpan
	CAPABILITY_FloatingPointCoordinates =	0x00000400,	//!< Handles floating point coordinates at device level.
	CAPABILITY_BitmapFillRGBA =				0x00000800,	//!< Implements its own optimized bitmap pattern fill for RGBA bitmaps in FillSpan
	CAPABILITY_SimpleBorder =				0x00001000,	//!< Implements its own simple border rendering with one pass
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CAPABILITY)
#endif

enum BASELINE {
	BASELINE_Unspecified		= 0,
	BASELINE_Straight 		= 0x00000001,
	BASELINE_Spline 			= 0x00000002,
	BASELINE_Nonconvex 		= 0x00000004
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(BASELINE)
#endif

enum TEXTBORDER {
	TEXTBORDER_None			= 0,
	TEXTBORDER_SingleLine	= 1,
	TEXTBORDER_DoubleLine	= 2
	};


//! Flags for DrawXxxxStyleSample()
enum STYLESAMPLEFLAGS {
	STYLESAMPLE_Default =			0x0000,
	STYLESAMPLE_ShrinkToFit =		0x0001,
	STYLESAMPLE_ForceToFit =		0x0002,
	STYLESAMPLE_ZigZag =				0x0004,	//!< /\/ line sample
	STYLESAMPLE_RoundRect =			0x0008,	//!< "rounded" rectangles
	STYLESAMPLE_NoScaleToMap =		0x0010,	//!< Disable scaling to map, use symbol/pattern design scale
	STYLESAMPLE_CenterSymbol =		0x0020	//!< Center symbol
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(STYLESAMPLEFLAGS)
#endif


enum EMBEDPERMISSION {
	EMBEDPERMISSION_All				= 0,
	EMBEDPERMISSION_Restricted		= 0x00000002,
	EMBEDPERMISSION_PreviewPrint	= 0x00000004,
	EMBEDPERMISSION_Editable		= 0x00000008,
	EMBEDPERMISSION_NoSubset		= 0x00000100
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(EMBEDPERMISSION)
#endif

enum INTERFACETEXTSTYLE {
	INTERFACETEXTSTYLE_Normal		= 0,
	INTERFACETEXTSTYLE_Bold			= 0x00000001,
	INTERFACETEXTSTYLE_Italic		= 0x00000002,
	INTERFACETEXTSTYLE_Fixed		= 0x00000004	// Fixed width font
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(INTERFACETEXTSTYLE)
#endif

enum ICONFLAGS {
	ICON_OriginTop					= 0x00000000,
	ICON_OriginLeft				= 0x00000000,
	ICON_OriginBottom				= 0x00000001,
	ICON_OriginRight				= 0x00000002,
	ICON_OriginCenterX			= 0x00000004,
	ICON_OriginCenterY			= 0x00000008,
	ICON_OriginTopLeft			= (ICON_OriginTop|ICON_OriginLeft),
	ICON_OriginTopCenter			= (ICON_OriginTop|ICON_OriginCenterX),
	ICON_OriginTopRight			= (ICON_OriginTop|ICON_OriginRight),
	ICON_OriginCenterLeft		= (ICON_OriginCenterY|ICON_OriginLeft),
	ICON_OriginCenter				= (ICON_OriginCenterY|ICON_OriginCenterX),
	ICON_OriginCenterRight		= (ICON_OriginCenterY|ICON_OriginRight),
	ICON_OriginBottomLeft		= (ICON_OriginBottom|ICON_OriginLeft),
	ICON_OriginBottomCenter		= (ICON_OriginBottom|ICON_OriginCenterX),
	ICON_OriginBottomRight		= (ICON_OriginBottom|ICON_OriginRight),
	ICON_Dimmed						= 0x00000010	//!< Draw "dimmed" (stippled) icon
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(ICONFLAGS)
#endif

enum BUILTINFILLSTYLE {
	BUILTINFILLSTYLE_Solid	= 0,
	BUILTINFILLSTYLE_50pct,			//!< 	Every other pixel in a checkerboard pattern
	BUILTINFILLSTYLE_OneIn2x2,		//!<		2x2 pattern with one pixel on
	BUILTINFILLSTYLE_OneIn4x4,		//!<		4x4 pattern with one pixel on
	BUILTINFILLSTYLE_OneIn6x6,		//!<		6x6 pattern with one pixel on
	BUILTINFILLSTYLE_OneIn8x8,		//!<		8x8 pattern with one pixel on
	BUILTINFILLSTYLE_Hatch8L,		//!< 45 degree hatch every 8 pixels (going down to the left)
	BUILTINFILLSTYLE_Hatch8R,		//!< 45 degree hatch every 8 pixels (going down to the right)

	};

//! These constants are defined by the TrueType spec and are used for TextGetTTFName()
enum TTFNAMEID {
	TTFNAMEID_Copyright		= 0,	//!< eg: "Copyright Apple Computer, Inc. 1992"
	TTFNAMEID_Family			= 1,	//!< eg: "New York"
	TTFNAMEID_Style			= 2,	//!< eg: "Bold"
	TTFNAMEID_ID				= 3,	//!< eg: "Apple Computer New York Bold version 1.0"
	TTFNAMEID_FullName		= 4,	//!< eg: "New York Bold"
	TTFNAMEID_Version			= 5,	//!< eg: "August 10, 1991, 1.08d21"
	TTFNAMEID_PostScript		= 6,	//!< eg: "Times-Bold"
	TTFNAMEID_Trademark		= 7,
	TTFNAMEID_Designer		= 8
	};


enum COMPONENT {
	COMPONENT_None = 0,
	COMPONENT_Red,
	COMPONENT_Green,
	COMPONENT_Blue,
	COMPONENT_Alpha
	};

} // End of namespace MGD


#endif	// INC_MI32_MG2ENUMS_H


