/**
 * \file mi32/recordbuffer.h
 * \brief RECORDBUFFER class definitions
 *
 * \if NODOC
 * $Id: recordbuffer.h_v 1.7 2012/09/14 13:53:20 mju Exp $
 *
 * $Log: recordbuffer.h_v $
 * Revision 1.7  2012/09/14 13:53:20  mju
 * Add next... methods to update offset.
 * Change get... array methods to return new offset.
 * Change all put... methods to return new offset.
 * Use uint32 for all size/offset/count parms.
 *
 * Revision 1.6  2012/09/11 19:56:04  mju
 * Add putU/INT8 with array.
 *
 * Revision 1.5  2012/09/06 15:10:20  mju
 * Add get/putu/int64 methods.
 *
 * Revision 1.4  2012/08/30 15:38:23  mju
 * Don't return value from getComposite.
 *
 * Revision 1.3  2012/08/30 15:35:32  mju
 * Make getComposite const.
 *
 * Revision 1.1  2009/03/11 15:45:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_RECORDBUFFER_H
#define	INC_MI32_RECORDBUFFER_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

//=====================================================================================================================

//! Base for record buffer classes.
class RECORDBUFFER {
	public:

		//! Clear data in record - does not change size.
		void Clear (
			) { memset(static_cast<UINT8*>(m_buffer),0,GetSize()); }

		//! Get non-const pointer to buffer memory.
		UINT8* GetPtr (
			) { return (m_buffer); }

		//! Get const pointer to buffer memory.
		const UINT8* GetPtr (
			) const { return (m_buffer); }

		//! Get record size in bytes.
		UINT32 GetSize (
			) const { return (m_buffer.GetNumItems()); }

		//! Get INT8 value from record.
		INT8 GetINT8 (
			UINT32 offset					//! Byte offset to value, not validated
			) const { return (static_cast<INT8>(m_buffer[offset])); }

		//! Get INT8 value from record.
		UINT8 GetUINT8 (
			UINT32 offset					//! Byte offset to value, not validated
			) const { return (m_buffer[offset]); }

		//! Get INT8 value from record and increment offset.
		INT8 NextINT8 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT8 result = GetINT8(offset); offset += sizeof(INT8); return (result); }

		//! Get UINT8 value from record and increment offset.
		UINT8 NextUINT8 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT8 result = GetUINT8(offset); offset += sizeof(UINT8); return (result); }

		//! Put INT8 value into record.
		UINT32 PutINT8 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT8 value
			) { m_buffer[offset] = static_cast<UINT8>(value); return (offset + sizeof(INT8)); }

		//! Put INT8 array values into record.
		UINT32 PutINT8 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT8 *values,
			UINT32 numvalues				//! Number of values
			) { memcpy(&m_buffer[offset],values,numvalues);  return (offset + numvalues * sizeof(INT8)); }

		//! Put UINT8 value into record.
		UINT32 PutUINT8 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT8 value
			) { m_buffer[offset] = value;  return (offset + sizeof(UINT8)); }

		//! Put UINT8 array values into record.
		UINT32 PutUINT8 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT8 *values,
			UINT32 numvalues				//! Number of values
			) { memcpy(&m_buffer[offset],values,numvalues);  return (offset + numvalues * sizeof(UINT8)); }

		//! Put string of single-byte characters into record, only copying characters up to but not including terminator.
		UINT32 PutString (
			UINT32 offset,					//! Byte offset, not validated
			const char *string			//! String to put in record
			) { UINT32 len = strlen(string); memcpy(&m_buffer[offset],string,len); return (offset + len); }

		//! Put string of single-byte characters into record with length limitation and fill.
		UINT32 PutString (
			UINT32 offset,					//! Byte offset, not validated
			UINT32 maxchars,				//! Maximum number of characters to copy
			const char *string,			//! String to put in record
			char fillchar = 0				//! Fill character if maxchars < string length
			) {
			UINT32 len = strlen(string);
			if (len < maxchars) {
				memcpy(&m_buffer[offset],string,len);
				memset(&m_buffer[offset+len],fillchar,maxchars-len);
				}
			else {
				memcpy(&m_buffer[offset],string,maxchars);
				}
			return (offset + maxchars);
			}

		//! Resize record,
		ERRVALUE Resize (
			UINT32 numbytes,						//!< New number of bytes
			bool keepold = true,					//!< Keep existing record entries
			bool clear = false					//!< Clear new record entries to zeros
			) { return (m_buffer.Resize(numbytes,keepold,clear)); }

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT8> m_buffer;
		#endif // GENERATING_DOXYGEN_OUTPUT

		RECORDBUFFER () { }

		~RECORDBUFFER () { }

	};

//=====================================================================================================================

//! Record buffer where values are stored with least-significant byte first (little-endian / Intel order).
//! This class is intended to be used to store data in a format for working with external files
//! where values are stored least-significant byte first and that may not be aligned.
class RECORDBUFFER_LSBFIRST : public RECORDBUFFER {
	public:

		//! Get INT16 value from record.
		INT16 GetINT16 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_HiLo
			return ((static_cast<INT16>(m_buffer[offset+0]) << 8) + m_buffer[offset+1]);
			#else
			return ((static_cast<INT16>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get INT16 array values from record.
		UINT32 GetINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT16 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT16));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT16));
			}

		//! Get UINT16 value from record.
		UINT16 GetUINT16 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_HiLo
			return ((static_cast<UINT16>(m_buffer[offset+0]) << 8) + m_buffer[offset+1]);
			#else
			return ((static_cast<UINT16>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get UINT16 array values from record.
		UINT32 GetUINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT16 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT16));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT16));
			}

		//! Get INT32 value from record.
		INT32 GetINT32 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_HiLo
			return ((static_cast<INT32>(m_buffer[offset+0]) << 24) + (static_cast<INT32>(m_buffer[offset+1]) << 16) + (static_cast<INT32>(m_buffer[offset+2]) << 8) + m_buffer[offset+3]);
			#else
			return ((static_cast<INT32>(m_buffer[offset+3]) << 24) + (static_cast<INT32>(m_buffer[offset+2]) << 16) + (static_cast<INT32>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get INT32 array values from record.
		UINT32 GetINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT32 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT32));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT32));
			}

		//! Get UINT32 value from record.
		UINT32 GetUINT32 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_HiLo
			return ((static_cast<UINT32>(m_buffer[offset+0]) << 24) + (static_cast<UINT32>(m_buffer[offset+1]) << 16) + (static_cast<UINT32>(m_buffer[offset+2]) << 8) + m_buffer[offset+3]);
			#else
			return ((static_cast<UINT32>(m_buffer[offset+3]) << 24) + (static_cast<UINT32>(m_buffer[offset+2]) << 16) + (static_cast<UINT32>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get UINT32 array values from record.
		UINT32 GetUINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT32 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT32));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT32));
			}

		//! Get INT64 value from record.
		INT64 GetINT64 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			INT64 result;
			memcpy(&result,&m_buffer[offset],sizeof(result));
			#ifdef BYTEORDER_HiLo
				SwapBytes(result);
			#endif
			return (result);
			}

		//! Get INT64 array values from record.
		UINT32 GetINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT64 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT64));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT64));
			}

		//! Get UINT64 value from record.
		UINT64 GetUINT64 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			UINT64 result;
			memcpy(&result,&m_buffer[offset],sizeof(result));
			#ifdef BYTEORDER_HiLo
				SwapBytes(result);
			#endif
			return (result);
			}

		//! Get UINT64 array values from record.
		UINT32 GetUINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT64 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT64));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT64));
			}

		//! Get float value from record.
		float GetFloat (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			float value;
			memcpy(&value,&m_buffer[offset],sizeof(float));
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			return (value);
			}

		//! Get float array values from record.
		UINT32 GetFloat (
			UINT32 offset,					//! Byte offset to first value, not validated
			float *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(float));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(float));
			}

		//! Get double value from record.
		double GetDouble (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			double value;
			memcpy(&value,&m_buffer[offset],sizeof(double));
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			return (value);
			}

		//! Get double array values from record.
		UINT32 GetDouble (
			UINT32 offset,					//! Byte offset to first value, not validated
			double *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(double));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(double));
			}

		//! Get composite value from record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Item is byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 GetComposite (
			UINT32 offset,					//! Byte offset to value, not validated
			_TT& value						//! Value returned
			) const {
			memcpy(&value,&m_buffer[offset],sizeof(_TT));
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			return (offset + sizeof(_TT));
			}

		//! Get composite array values from record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Items are byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 GetComposite (
			UINT32 offset,					//! Byte offset to first value, not validated
			_TT *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(float));
			#ifdef BYTEORDER_HiLo
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(_TT));
			}

		//! Get INT16 value from record and increment offset.
		INT16 NextINT16 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT16 result = GetINT16(offset); offset += sizeof(INT16); return (result); }

		//! Get UINT16 value from record and increment offset.
		UINT16 NextUINT16 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT16 result = GetUINT16(offset); offset += sizeof(UINT16); return (result); }

		//! Get INT32 value from record and increment offset.
		INT32 NextINT32 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT32 result = GetINT32(offset); offset += sizeof(INT32); return (result); }

		//! Get UINT32 value from record and increment offset.
		UINT32 NextUINT32 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT32 result = GetUINT32(offset); offset += sizeof(UINT32); return (result); }

		//! Get INT64 value from record and increment offset.
		INT64 NextINT64 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT64 result = GetINT64(offset); offset += sizeof(INT64); return (result); }

		//! Get UINT64 value from record and increment offset.
		UINT64 NextUINT64 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT64 result = GetUINT64(offset); offset += sizeof(UINT64); return (result); }

		//! Get float value from record and increment offset.
		float NextFloat (
			UINT32& offset					//!< Byte offset passed/returned
			) const { float result = GetFloat(offset); offset += sizeof(float); return (result); }

		//! Get double value from record and increment offset.
		double NextDouble (
			UINT32& offset					//!< Byte offset passed/returned
			) const { double result = GetDouble(offset); offset += sizeof(double); return (result); }

		//! Put INT16 value into record.
		UINT32 PutINT16 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT16 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT16));
			return (offset + sizeof(INT16));
			}

		//! Put INT16 array values into record.
		UINT32 PutINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT16 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT16));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<INT16*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT16));
			}

		//! Put UINT16 value into record.
		UINT32 PutUINT16 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT16 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT16));
			return (offset + sizeof(UINT16));
			}

		//! Put UINT16 array values into record.
		UINT32 PutUINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT16 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT16));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<UINT16*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT16));
			}

		//! Put INT32 value into record.
		UINT32 PutINT32 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT32 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT32));
			return (offset + sizeof(INT32));
			}

		//! Put INT32 array values into record.
		UINT32 PutINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT32 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT32));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<INT32*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT32));
			}

		//! Put UINT32 value into record.
		UINT32 PutUINT32 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT32 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT32));
			return (offset + sizeof(UINT32));
			}

		//! Put UINT32 array values into record.
		UINT32 PutUINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT32 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT32));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<UINT32*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT32));
			}

		//! Put INT64 value into record.
		UINT32 PutINT64 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT64 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT64));
			return (offset + sizeof(INT64));
			}

		//! Put INT64 array values into record.
		UINT32 PutINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT64 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT64));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<INT64*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT64));
			}

		//! Put UINT64 value into record.
		UINT32 PutUINT64 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT64 value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT64));
			return (offset + sizeof(UINT64));
			}

		//! Put UINT64 array values into record.
		UINT32 PutUINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT64 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT64));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<UINT64*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT64));
			}

		//! Put float value into record.
		UINT32 PutFloat (
			UINT32 offset,					//! Byte offset to value, not validated
			float value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(float));
			return (offset + sizeof(float));
			}

		//! Put float array values into record.
		UINT32 PutFloat (
			UINT32 offset,					//! Byte offset to first value, not validated
			const float *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(float));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<float*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(float));
			}

		//! Put double value into record.
		UINT32 PutDouble (
			UINT32 offset,					//! Byte offset to value, not validated
			double value
			) {
			#ifdef BYTEORDER_HiLo
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(double));
			return (offset + sizeof(double));
			}

		//! Put double array values into record.
		UINT32 PutDouble (
			UINT32 offset,					//! Byte offset to first value, not validated
			const double *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(double));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<double*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(double));
			}

		//! Put composite value into record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Item is byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 PutComposite (
			UINT32 offset,					//! Byte offset to value, not validated
			const _TT& value
			) {
			memcpy(&m_buffer[offset],&value,sizeof(_TT));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<_TT&>(&m_buffer[offset]));
			#endif
			return (offset + sizeof(_TT));
			}

		//! Put composite array values into record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Items are byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 PutComposite (
			UINT32 offset,					//! Byte offset to first value, not validated
			const _TT *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(_TT));
			#ifdef BYTEORDER_HiLo
				SwapBytes(reinterpret_cast<_TT*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(_TT));
			}

	};

//=====================================================================================================================

//! Record buffer where values are stored with most-significant byte first (big-endian / Network order).
//! This class is intended to be used to store data in a format for working with external files
//! where values are stored most-significant byte first and that may not be aligned.
class RECORDBUFFER_MSBFIRST : public RECORDBUFFER {
	public:

		//! Get INT16 value from record.
		INT16 GetINT16 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_LoHi
			return ((static_cast<INT16>(m_buffer[offset+0]) << 8) + m_buffer[offset+1]);
			#else
			return ((static_cast<INT16>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get INT16 array values from record.
		UINT32 GetINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT16 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT16));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT16));
			}

		//! Get UINT16 value from record.
		UINT16 GetUINT16 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_LoHi
			return ((static_cast<UINT16>(m_buffer[offset+0]) << 8) + m_buffer[offset+1]);
			#else
			return ((static_cast<UINT16>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get UINT16 array values from record.
		UINT32 GetUINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT16 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT16));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT16));
			}

		//! Get INT32 value from record.
		INT32 GetINT32 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_LoHi
			return ((static_cast<INT32>(m_buffer[offset+0]) << 24) + (static_cast<INT32>(m_buffer[offset+1]) << 16) + (static_cast<INT32>(m_buffer[offset+2]) << 8) + m_buffer[offset+3]);
			#else
			return ((static_cast<INT32>(m_buffer[offset+3]) << 24) + (static_cast<INT32>(m_buffer[offset+2]) << 16) + (static_cast<INT32>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get INT32 array values from record.
		UINT32 GetINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT32 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT32));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT32));
			}

		//! Get UINT32 value from record.
		UINT32 GetUINT32 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			#ifdef BYTEORDER_LoHi
			return ((static_cast<UINT32>(m_buffer[offset+0]) << 24) + (static_cast<UINT32>(m_buffer[offset+1]) << 16) + (static_cast<UINT32>(m_buffer[offset+2]) << 8) + m_buffer[offset+3]);
			#else
			return ((static_cast<UINT32>(m_buffer[offset+3]) << 24) + (static_cast<UINT32>(m_buffer[offset+2]) << 16) + (static_cast<UINT32>(m_buffer[offset+1]) << 8) + m_buffer[offset+0]);
			#endif
			}

		//! Get UINT32 array values from record.
		UINT32 GetUINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT32 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT32));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT32));
			}

		//! Get INT64 value from record.
		INT64 GetINT64 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			INT64 result;
			memcpy(&result,&m_buffer[offset],sizeof(result));
			#ifdef BYTEORDER_LoHi
				SwapBytes(result);
			#endif
			return (result);
			}

		//! Get INT64 array values from record.
		UINT32 GetINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			INT64 *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(INT64));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(INT64));
			}

		//! Get UINT64 value from record.
		UINT64 GetUINT64 (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			UINT64 result;
			memcpy(&result,&m_buffer[offset],sizeof(result));
			#ifdef BYTEORDER_LoHi
				SwapBytes(result);
			#endif
			return (result);
			}

		//! Get UINT64 array values from record.
		UINT32 GetUINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			UINT64 *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(UINT64));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT64));
			}

		//! Get float value from record.
		float GetFloat (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			float value;
			memcpy(&value,&m_buffer[offset],sizeof(float));
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			return (value);
			}

		//! Get float array values from record.
		UINT32 GetFloat (
			UINT32 offset,					//! Byte offset to first value, not validated
			float *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(float));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(float));
			}

		//! Get double value from record.
		double GetDouble (
			UINT32 offset					//! Byte offset to value, not validated
			) const {
			double value;
			memcpy(&value,&m_buffer[offset],sizeof(double));
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			return (value);
			}

		//! Get double array values from record.
		UINT32 GetDouble (
			UINT32 offset,					//! Byte offset to first value, not validated
			double *values,				//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(double));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(double));
			}

		//! Get composite value from record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Item is byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 GetComposite (
			UINT32 offset,					//! Byte offset to value, not validated
			_TT& value						//! Value returned
			) const {
			memcpy(&value,&m_buffer[offset],sizeof(_TT));
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			return (offset + sizeof(_TT));
			}

		//! Get composite array values from record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Items are byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 GetComposite (
			UINT32 offset,					//! Byte offset to first value, not validated
			_TT *values,					//! Array of values returned
			UINT32 numvalues					//! Number of values
			) const {
			memcpy(values,&m_buffer[offset],numvalues*sizeof(float));
			#ifdef BYTEORDER_LoHi
				SwapBytes(values,numvalues);
			#endif
			return (offset + numvalues * sizeof(_TT));
			}

		//! Get INT16 value from record and increment offset.
		INT16 NextINT16 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT16 result = GetINT16(offset); offset += sizeof(INT16); return (result); }

		//! Get UINT16 value from record and increment offset.
		UINT16 NextUINT16 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT16 result = GetUINT16(offset); offset += sizeof(UINT16); return (result); }

		//! Get INT32 value from record and increment offset.
		INT32 NextINT32 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT32 result = GetINT32(offset); offset += sizeof(INT32); return (result); }

		//! Get UINT32 value from record and increment offset.
		UINT32 NextUINT32 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT32 result = GetUINT32(offset); offset += sizeof(UINT32); return (result); }

		//! Get INT64 value from record and increment offset.
		INT64 NextINT64 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { INT64 result = GetINT64(offset); offset += sizeof(INT64); return (result); }

		//! Get UINT64 value from record and increment offset.
		UINT64 NextUINT64 (
			UINT32& offset					//!< Byte offset passed/returned
			) const { UINT64 result = GetUINT64(offset); offset += sizeof(UINT64); return (result); }

		//! Get float value from record and increment offset.
		float NextFloat (
			UINT32& offset					//!< Byte offset passed/returned
			) const { float result = GetFloat(offset); offset += sizeof(float); return (result); }

		//! Get double value from record and increment offset.
		double NextDouble (
			UINT32& offset					//!< Byte offset passed/returned
			) const { double result = GetDouble(offset); offset += sizeof(double); return (result); }

		//! Put INT16 value into record.
		UINT32 PutINT16 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT16 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT16));
			return (offset + sizeof(INT16));
			}

		//! Put INT16 array values into record.
		UINT32 PutINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT16 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT16));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<INT16*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT16));
			}

		//! Put UINT16 value into record.
		UINT32 PutUINT16 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT16 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT16));
			return (offset + sizeof(UINT16));
			}

		//! Put UINT16 array values into record.
		UINT32 PutUINT16 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT16 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT16));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<UINT16*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT16));
			}

		//! Put INT32 value into record.
		UINT32 PutINT32 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT32 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT32));
			return (offset + sizeof(INT32));
			}

		//! Put INT32 array values into record.
		UINT32 PutINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT32 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT32));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<INT32*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT32));
			}

		//! Put UINT32 value into record.
		UINT32 PutUINT32 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT32 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT32));
			return (offset + sizeof(UINT32));
			}

		//! Put UINT32 array values into record.
		UINT32 PutUINT32 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT32 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT32));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<UINT32*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT32));
			}

		//! Put INT64 value into record.
		UINT32 PutINT64 (
			UINT32 offset,					//! Byte offset to value, not validated
			INT64 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(INT64));
			return (offset + sizeof(INT64));
			}

		//! Put INT64 array values into record.
		UINT32 PutINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const INT64 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(INT64));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<INT64*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(INT64));
			}

		//! Put UINT64 value into record.
		UINT32 PutUINT64 (
			UINT32 offset,					//! Byte offset to value, not validated
			UINT64 value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(UINT64));
			return (offset + sizeof(UINT64));
			}

		//! Put UINT64 array values into record.
		UINT32 PutUINT64 (
			UINT32 offset,					//! Byte offset to first value, not validated
			const UINT64 *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(UINT64));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<UINT64*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(UINT64));
			}

		//! Put float value into record.
		UINT32 PutFloat (
			UINT32 offset,					//! Byte offset to value, not validated
			float value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(float));
			return (offset + sizeof(float));
			}

		//! Put float array values into record.
		UINT32 PutFloat (
			UINT32 offset,					//! Byte offset to first value, not validated
			const float *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(float));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<float*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(float));
			}

		//! Put double value into record.
		UINT32 PutDouble (
			UINT32 offset,					//! Byte offset to value, not validated
			double value
			) {
			#ifdef BYTEORDER_LoHi
				SwapBytes(value);
			#endif
			memcpy(&m_buffer[offset],&value,sizeof(double));
			return (offset + sizeof(double));
			}

		//! Put double array values into record.
		UINT32 PutDouble (
			UINT32 offset,					//! Byte offset to first value, not validated
			const double *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(double));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<double*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(double));
			}

		//! Put composite value into record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Item is byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 PutComposite (
			UINT32 offset,					//! Byte offset to value, not validated
			const _TT& value
			) {
			memcpy(&m_buffer[offset],&value,sizeof(_TT));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<_TT&>(&m_buffer[offset]));
			#endif
			return (offset + sizeof(_TT));
			}

		//! Put composite array values into record.
		//! There must be a SwapBytes(...) method defined for the composite type.
		//! Items are byte-copied, so check alignment issues carefully.
		template <typename _TT> UINT32 PutComposite (
			UINT32 offset,					//! Byte offset to first value, not validated
			const _TT *values,
			UINT32 numvalues					//! Number of values
			) {
			memcpy(&m_buffer[offset],values,numvalues*sizeof(_TT));
			#ifdef BYTEORDER_LoHi
				SwapBytes(reinterpret_cast<_TT*>(&m_buffer[offset]),numvalues);
			#endif
			return (offset + numvalues * sizeof(_TT));
			}

	};

//=====================================================================================================================

#endif	// INC_MI32_RECORDBUFFER_H
