/**
 * \file mi32/regiongen.h
 * \brief REGIONGEN class definitions
 *
 * \if NODOC
 * $Id: regiongen.h_v 1.6 2011/11/04 13:18:34 mju Exp $
 *
 * $Log: regiongen.h_v $
 * Revision 1.6  2011/11/04 13:18:34  mju
 * Remove redundant hdrs.
 *
 * Revision 1.5  2004/09/24 15:01:24  mju
 * Add setBoundaryPoints for seeded.
 *
 * Revision 1.4  2004/09/24 13:20:20  mju
 * Add RASTER_SEEDED.
 *
 * Revision 1.3  2004/09/23 20:49:31  mju
 * Add maxdifference setting for raster_values.
 *
 * Revision 1.1  2004/09/23 14:34:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_REGIONGEN_H
#define	INC_MI32_REGIONGEN_H

#ifndef	INC_MI32_RASTERGEN_H
#include <mi32/rastergen.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class REGION2D;
namespace RVC {
	class GEOSPATIAL;
	}
#endif

//=====================================================================================================================

//! Base for region generation.
class REGIONGEN {
	public:

		//! Default constructor.
		REGIONGEN ();

		//! Destructor.
		virtual ~REGIONGEN ();

		//! Compute region.
		ERRVALUE Compute (
			REGION2D& RegionResult							//!< Result region returned
			) const { return (v_Compute(RegionResult)); }

		bool IsReturnMapCRS (
			) const { return (m_ReturnMapCRS); }

		//! Set whether to return region in map coordinates.
		//! Default is to return in map coordinates.
		//! Alternative is to return in source object coordinates.
		void SetReturnMapCRS (
			bool ReturnMapCRS									//!< True for map coordinates, false for source object coordinates
			) { m_ReturnMapCRS = ReturnMapCRS; }

	protected:

		//! Helper to convert region to map CRS.
		//! Checks if conversion is requested.
		ERRVALUE ConvertToMapCRS (
			const RVC::GEOSPATIAL& ObjSource,			//!< Open 'source' object to obtain georeference from
			REGION2D& RegionResult							//!< Result region returned
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_ReturnMapCRS;
		#endif

		//! Override to compute region.
		virtual ERRVALUE v_Compute (
			REGION2D& RegionResult							//!< Result region returned
			) const = 0;
	};


//=====================================================================================================================

//! Generate region from binary raster.
class REGIONGEN_RASTER_BINARY : public REGIONGEN {
	public:

		//! Constructor.
		REGIONGEN_RASTER_BINARY ();

		//! Destructor.
		~REGIONGEN_RASTER_BINARY ();

		//! Get source raster OBJITEM.
		const RVC::OBJITEM& GetSource (
			) const { return (m_ObjItemSource); }

		//! Set source raster OBJITEM.
		void SetSource (
			const RVC::OBJITEM& ObjItemSource			//!< Item for source raster
			) { m_ObjItemSource = ObjItemSource; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemSource;

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};


//=====================================================================================================================

//! Generate region around non-null raster cells.
class REGIONGEN_RASTER_NONULL : public REGIONGEN {
	public:

		//! Constructor.
		REGIONGEN_RASTER_NONULL ();

		//! Destructor.
		~REGIONGEN_RASTER_NONULL ();

		//! Get source raster OBJITEM.
		const RVC::OBJITEM& GetSource (
			) const { return (m_ObjItemSource); }

		//! Set source raster OBJITEM.
		void SetSource (
			const RVC::OBJITEM& ObjItemSource			//!< Item for source raster
			) { m_ObjItemSource = ObjItemSource; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemSource;

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};


//=====================================================================================================================

//! Generate region from raster value(s).
class REGIONGEN_RASTER_VALUES : public REGIONGEN {
	public:

		//! Constructor.
		REGIONGEN_RASTER_VALUES ();

		//! Destructor.
		~REGIONGEN_RASTER_VALUES ();

		//! Get maximum difference between specified and actual cell value.
		double GetMaxDifference (
			) const { return (m_MaxDifference); }

		//! Get source raster OBJITEM.
		const RVC::OBJITEM& GetSource (
			) const { return (m_ObjItemSource); }

		//! Get values to trace.
		const DOUBLE_ARRAY<double>& GetValues (
			) const { return (m_Values); }

		//! Set maximum difference between specified and actual cell value.
		void SetMaxDifference (
			double MaxDifference
			) { m_MaxDifference = MaxDifference; }

		//! Set source raster OBJITEM.
		void SetSource (
			const RVC::OBJITEM& ObjItemSource			//!< Item for source raster
			) { m_ObjItemSource = ObjItemSource; }

		//! Set single value to trace.
		//! Replaces any previously set values.
		void SetValue (
			double value
			);

		//! Set multiple values to trace.
		//! Replaces any previously set values.
		void SetValues (
			const DOUBLE_ARRAY<double>& values
			);

		//! Set multiple values to trace.
		//! Replaces any previously set values.
		void SetValues (
			const double* values,
			int numvalues
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemSource;
		DOUBLE_ARRAY<double> m_Values;
		double m_MaxDifference;

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};


//=====================================================================================================================

//! Generate region from raster range(s).
class REGIONGEN_RASTER_RANGES : public REGIONGEN {
	public:

		//! Constructor.
		REGIONGEN_RASTER_RANGES ();

		//! Destructor.
		~REGIONGEN_RASTER_RANGES ();

		//! Get ranges to trace.
		const DOUBLE_ARRAY<DOUBLE_RANGE>& GetRanges (
			) const { return (m_Ranges); }

		//! Get source raster OBJITEM.
		const RVC::OBJITEM& GetSource (
			) const { return (m_ObjItemSource); }

		//! Set single range to trace.
		//! Replaces any previously set ranges.
		void SetRange (
			const DOUBLE_RANGE& range
			);

		//! Set single range to trace.
		//! Replaces any previously set ranges.
		void SetRange (
			double min,
			double max
			);

		//! Set ranges to trace.
		//! Replaces any previously set ranges.
		void SetRanges (
			const DOUBLE_ARRAY<DOUBLE_RANGE>& ranges
			);

		//! Set source raster OBJITEM.
		void SetSource (
			const RVC::OBJITEM& ObjItemSource			//!< Item for source raster
			) { m_ObjItemSource = ObjItemSource; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemSource;
		DOUBLE_ARRAY<DOUBLE_RANGE> m_Ranges;

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};


//=====================================================================================================================

//! Generate region from raster using seed point(s) for area/boundary trace.
class REGIONGEN_RASTER_SEEDED : public REGIONGEN {
	public:

		//! Seed 'filling' method.
		enum METHOD {
			METHOD_Solid,
			METHOD_Boundary
			};

		//! Constructor.
		REGIONGEN_RASTER_SEEDED ();

		//! Destructor.
		~REGIONGEN_RASTER_SEEDED ();

		//! Get boundary point array.
		const DOUBLE_ARRAY<DPOINT2D>& GetBoundaryPoints (
			) const { return (m_BoundaryPoints); }

		//! Get boundary value array.
		const DOUBLE_ARRAY<double>& GetBoundaryValues (
			) const { return (m_BoundaryValues); }

		//! Get maximum allowed difference for value comparison.
		double GetMaxDifference (
			) const { return (m_MaxDifference); }

		//! Get boundary/fill method.
		METHOD GetMethod (
			) const { return (m_Method); }

		//! Get coordinate reference system for seed and boundary points.
		const SPATREF::COORDREFSYS& GetPointCRS (
			) const { return (m_PointCRS); }

		//! Get seed points.
		const DOUBLE_ARRAY<DPOINT2D>& GetSeedPoints (
			) const { return (m_SeedPoints); }

		//! Get source raster OBJITEM.
		const RVC::OBJITEM& GetSource (
			) const { return (m_ObjItemSource); }

		//! Set boundary points.
		//! Replaces any previously set values or points.
		//! Automatically sets method to METHOD_Boundary if array is non-empty.
		void SetBoundaryPoints (
			const DOUBLE_ARRAY<DPOINT2D>& BoundaryPoints
			);

		//! Set single boundary value.
		//! Replaces any previously set values or points.
		//! Automatically sets method to METHOD_Boundary.
		void SetBoundaryValue (
			double value
			);

		//! Set multiple boundary value.
		//! Replaces any previously set values or points.
		//! Automatically sets method to METHOD_Boundary if array is non-empty.
		void SetBoundaryValues (
			const DOUBLE_ARRAY<double>& values
			);

		//! Set maximum difference for value (area or boundary) comparison.
		//! If not set then will use exact match.
		void SetMaxDifference (
			double MaxDifference
			) { m_MaxDifference = MaxDifference; }

		//! Set method for region generation.
		//! If not set will do Solid trace unless boundary points are set.
		void SetMethod (
			METHOD Method
			) { m_Method = Method; }

		//! Set coordinate reference system for seed and boundary points.
		//! If not set or undefined then points must be in source raster coordinates.
		void SetPointCRS (
			const SPATREF::COORDREFSYS& PointCRS
			) { m_PointCRS = PointCRS; }

		//! Set single seed point.
		void SetSeedPoint (
			const DPOINT2D& SeedPoint
			);

		//! Set multiple seed points.
		void SetSeedPoints (
			const DOUBLE_ARRAY<DPOINT2D>& SeedPoints
			);

		//! Set source raster OBJITEM.
		void SetSource (
			const RVC::OBJITEM& ObjItemSource			//!< Item for source raster
			) { m_ObjItemSource = ObjItemSource; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class JOB;

		RVC::OBJITEM m_ObjItemSource;
		DOUBLE_ARRAY<DPOINT2D> m_SeedPoints;
		SPATREF::COORDREFSYS m_PointCRS;
		DOUBLE_ARRAY<DPOINT2D> m_BoundaryPoints;
		DOUBLE_ARRAY<double> m_BoundaryValues;
		double m_MaxDifference;
		METHOD m_Method;

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};


//=====================================================================================================================

//! Generate region from viewshed.
class REGIONGEN_VIEWSHED : public REGIONGEN, public RASTERGEN_VIEWSHED {
	public:

		REGIONGEN_VIEWSHED (
			);

		virtual ~REGIONGEN_VIEWSHED (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		ERRVALUE v_Compute (
			REGION2D& RegionResult
			) const;

		#endif
	};
	
//=====================================================================================================================

#endif	// INC_MI32_REGIONGEN_H
