/**
 * \file statunivar.h <mi32/statunivar.h>
 * \brief STAT_UNIVARIATE class definitions.
 *
 * \if NODOC
 * $Id: statunivar.h_v 1.18 2011/10/11 21:42:55 mju Exp $
 *
 * $Log: statunivar.h_v $
 * Revision 1.18  2011/10/11 21:42:55  mju
 * Remove redundant hdr.
 *
 * Revision 1.17  2011/07/22 17:13:09  mju
 * Change accumulateCounted to use uint64.
 *
 * Revision 1.16  2011/07/21 20:56:50  mju
 * Add get/setFromXML.
 *
 * Revision 1.15  2010/06/10 15:41:23  mju
 * Add findBinIdxPctAbove/Below.
 *
 * Revision 1.14  2010/05/20 17:24:33  mju
 * Add getDisplay and optional numtype parm to set().
 *
 * Revision 1.13  2009/12/11 15:27:26  mju
 * Change init from existing to include numtype parm.
 *
 * Revision 1.12  2009/12/04 14:49:14  mju
 * Add initialize based on existing stats.
 *
 * Revision 1.11  2009/01/15 16:57:36  mju
 * Add getNumType.
 *
 * Revision 1.10  2008/04/18 17:41:44  mju
 * Add setAllInvalid.
 *
 * Revision 1.9  2007/11/05 17:57:20  mju
 * Add accumulateCounted.
 *
 * Revision 1.8  2007/09/13 21:34:15  mju
 * Add computeBinRange.
 *
 * Revision 1.7  2007/09/10 20:56:23  mju
 * Add accumulate with stats.
 *
 * Revision 1.6  2007/09/10 19:24:59  mju
 * Add isValid and clear methods.
 *
 * Revision 1.5  2006/01/30 21:30:14  mju
 * Add getBinIndexLimit.
 *
 * Revision 1.4  2005/10/27 20:25:06  mju
 * Support weighted accumlation.
 *
 * Revision 1.3  2005/10/12 14:58:37  mju
 * Add getMedian/ModeBinIndex.
 *
 * Revision 1.1  2005/09/01 15:07:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STATUNIVAR_H
#define	INC_MI32_STATUNIVAR_H

#ifndef	INC_MI32_NUMTYPE_H
#include <mi32/numtype.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLNODE;
#endif

//=====================================================================================================================

//! Container for accumulating and computing univariate statistics.
//! Statistics include minimum, maximum, mean, standard deviation and optional histogram.
class CLASSLIBEXPORT STAT_UNIVARIATE {
	public:

		//! Default constructor.
		STAT_UNIVARIATE ();

		//! Destructor.
		~STAT_UNIVARIATE ();

		//! Accumulate samples for computing statistics.
		ERRVALUE Accumulate (
			UINT32 NumValues,						//!< Number of sample values
			const void *pValues,					//!< Pointer to sample values, must match numeric type specified in Initialize
			const UINT8 *pValidity = 0			//!< Pointer to validity, if NULL all samples considered valid
			);

		//! Accumulate from another set of statistics.
		ERRVALUE Accumulate (
			const STAT_UNIVARIATE& stats		//!< Stats to add in
			);

		//! Accumulate single sample with count.
		ERRVALUE AccumulateCounted (
			const void *pValue,					//!< Pointer to sample value, must match numeric type specified in Initialize, NULL for invalid
			UINT64 count							//!< Count of values
			);

		//! Accumulate single sample with weight.
		ERRVALUE AccumulateWeighted (
			const void *pValue,					//!< Pointer to sample value, must match numeric type specified in Initialize
			double weight							//!< Weight, 1.0 same as whole sample
			);

		//! Clear all statistics, reverting to "invalid" state.
		void Clear (
			);

		//! Compute center value for specified bin.
		double ComputeBinCenter (
			INT32 BinIndex							//!< Bin index
			) const;

		//! Compute bin index from specified value.
		INT32 ComputeBinIndex (
			double value
			) const;

		//! Compute bin index from specified value with value forced to table limit.
		UINT32 ComputeBinIndexLimit (
			double value
			) const;

		//! Compute range of values counted in specified bin.
		void ComputeBinRange (
			INT32 BinIndex,						//!< Bin index
			DOUBLE_RANGE& BinRange				//!< Range of values for bin returned
			) const;

		//! Finalize computation of statistics.
		ERRVALUE Finalize ();

		//! Find bin index at most specified percentage of counts are above.
		//! @return Bin index or error < 0
		INT32 FindBinIndexPctAbove (
			double pct								//!< Percentage (0-100)
			) const;

		//! Find bin index at most specified percentage of counts are below.
		//! @return Bin index or error < 0
		INT32 FindBinIndexPctBelow (
			double pct								//!< Percentage (0-100)
			) const;

		//! Get count of samples in specified bin.
		//! If any weighted will be rounded up, use GetBinCountWeighted for weighted total.
		UINT64 GetBinCount (
			INT32 BinIndex							//!< Bin index
			) const;

		//! Get count of samples in specified bin with partial counts for weighted.
		double GetBinCountWeighted (
			INT32 BinIndex							//!< Bin index
			) const;

		//! Get range for histogram bins.
		const DOUBLE_RANGE& GetBinRange (
			) const { return (m_BinRange); }

		//! Get width of specified bin.
		double GetBinWidth (
			INT32 BinIndex = 0					//!< Bin index
			) const;

		//! Get count of valid samples above maximum bin.
		//! If any weighted will be rounded up, use GetCountAboveMaxBinWeighted for weighted total.
		UINT64 GetCountAboveMaxBin (
			) const;

		//! Get count of valid samples above maximum bin with partial counts for weighted.
		double GetCountAboveMaxBinWeighted (
			) const;

		//! Get count of valid samples below minimum bin.
		//! If any weighted will be rounded up, use GetCountBelowMinBinWeighted for weighted total.
		UINT64 GetCountBelowMinBin (
			) const;

		//! Get count of valid samples below minimum bin with partial counts for weighted.
		double GetCountBelowMinBinWeighted (
			) const;

		//! Get count of invalid samples.
		UINT64 GetCountInvalid (
			) const { return (m_CountInvalid); }

		//! Get count of valid samples.
		//! If any weighted will be rounded up, use GetCountValidWeighted for weighted total.
		UINT64 GetCountValid (
			) const;

		//! Get count of valid samples with partial counts for weighted.
		double GetCountValidWeighted (
			) const;

		//! Get display string for statistics (without bin counts).
		MISTRING GetDisplayStats (
			) const;

		//! Get mean sample value.
		double GetMean (
			) const { return (m_Mean); }

		//! Get index for median bin.
		INT32 GetMedianBinIndex (
			) const { return (m_MedianBinIndex); }

		//! Get index for mode bin.
		INT32 GetModeBinIndex (
			) const { return (m_ModeBinIndex); }

		//! Get number of bins.
		UINT32 GetNumBins (
			) const { return (m_BinCounts.GetNumItems()); }

		//! Get sample numeric type.
		NUMTYPE GetNumType (
			) const { return (m_NumType); }

		//! Get standard deviation.
		double GetStdDev (
			) const { return (m_StdDev); }

		//! Get sum of squares.
		double GetSum2 (
			) const { return (m_Sum2); }

		//! Get range of accumulated values.
		const DOUBLE_RANGE& GetValueRange (
			) const { return (m_ValueRange); }

		//! Get XMLNODE representation for saving.
		XMLNODE* GetXML (
			XMLNODE *parentnode,					//!< Parent node
			const char *nodename					//!< Node name
			) const;

		//! Determine if has equal bin widths.
		bool HasEqualBinWidths (
			) const { return (true); }

		//! Initialize for accumulation and statistics computation, including histogram with equal bin widths.
		//! Note that for integer types, the bin width will also be an integer, thus the last bin may have a smaller width.
		ERRVALUE Initialize (
			NUMTYPE NumType,						//!< Sample numeric type
			UINT32 NumBins,						//!< Number of bins to allocate, if 0 will determine from bin range and width
			double MinBin,							//!< Minimum bin value
			double MaxBin,							//!< Maximum bin value
			double BinWidth = 0					//!< Bin width, if 0 will determine from number of bins and range
			);

		//! Initialize for accumulation and statistics computation, using same bin settings as existing instance.
		ERRVALUE Initialize (
			NUMTYPE NumType,						//!< Sample numeric type
			const STAT_UNIVARIATE& existing	//!< Existing statistics to initialize bin settings from
			) { return (Initialize(NumType,existing.m_BinCounts.GetNumItems(),existing.m_BinRange.GetMinimum(),existing.m_BinRange.GetMaximum(),existing.m_BinWidth)); }

		//! Determine if has valid statistics.
		bool IsValid (
			) const { return (m_State == STATE_Valid); }

		//! Set all statistics.
		ERRVALUE Set (
			UINT64 CountValid,						//!< Count of valid samples
			UINT64 CountBelowMinBin,				//!< Count of samples below minimum bin
			UINT64 CountAboveMaxBin,				//!< Count of samples above maximum bin
			UINT64 CountInvalid,						//!< Count of invalid samples
			double Mean,								//!< Mean
			double Sum2,								//!< Sum of squares of sample values
			const DOUBLE_RANGE& ValueRange,		//!< Range of sample values
			const DOUBLE_RANGE& BinRange,			//!< Range of bin values
			double BinWidth,							//!< Bin width
			const void *BinCounts,					//!< Bin count array
			UINT32 NumBins,							//!< Number of bins
			int BinBytes,								//!< Bin size in bytes (1, 2, 4 or 8)
			NUMTYPE NumType = NUMTYPE_Invalid	//!< Numeric data type
			);

		//! Set to having only "invalid" data samples.
		void SetAllInvalid (
			UINT64 CountInvalid						//!< Count of invalid samples
			);

		//! Set from XML.
		//! Will be cleared on failure.
		ERRVALUE SetFromXML (
			const XMLNODE *xmlnode					//!< Node to set from
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum STATE {
			STATE_Invalid,
			STATE_Accumulating,
			STATE_Valid
			};

		STATE m_State;
		NUMTYPE m_NumType;
		DOUBLE_RANGE m_ValueRange;
		DOUBLE_RANGE m_BinRange;
		double m_BinWidth;					// Bin width if equal
		double m_Mean;							// Mean
		double m_StdDev;						// Standard deviation
		double m_Sum2;							// Sum of squares
		UINT64 m_CountBelowMinBin;
		UINT64 m_CountAboveMaxBin;
		UINT64 m_CountValid;
		UINT64 m_CountInvalid;
		double m_WeightedBelowMinBin;
		double m_WeightedAboveMaxBin;
		double m_WeightedTotal;
		INT32 m_MedianBinIndex;
		INT32 m_ModeBinIndex;
		DOUBLE_ARRAY<UINT64> m_BinCounts;
		DOUBLE_ARRAY<double> m_BinWeighted;
//		DOUBLE_ARRAY<double> m_BinCenters;
//		DOUBLE_ARRAY<double> m_BinWidths;

		void ClearLow ();
		void ClearWeighted ();
		void ComputeMedianAndMode ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#undef	CLASSLIBEXPORT

#endif	 // INC_MI32_STATUNIVAR_H
