/**
 * \file mi32/stereoshift.h
 * \brief STEREOSHIFT definitions
 *
 * \if NODOC
 * $Id: stereoshift.h_v 1.7 2012/09/18 13:49:19 mju Exp $
 *
 * $Log: stereoshift.h_v $
 * Revision 1.7  2012/09/18 13:49:19  mju
 * Dox ctor fix.
 *
 * Revision 1.6  2012/04/04 20:47:18  mju
 * Store absolute scale and have separate methods for left/right point shift.
 *
 * Revision 1.5  2012/03/30 21:13:35  mju
 * Make calcPointSingle inline.
 *
 * Revision 1.4  2012/03/30 19:59:46  mju
 * Add calcPoint/Polyline methods.
 *
 * Revision 1.3  2012/03/30 17:28:30  mju
 * Use stereoview instead of bools.
 *
 * Revision 1.2  2012/03/30 14:44:34  mju
 * Add isValid.
 *
 * Revision 1.1  2012/03/30 14:27:09  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_STEREOSHIFT_H
#define	INC_MI32_STEREOSHIFT_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_STEREOVIEW_H
#include <mi32/stereoview.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
#endif


//! Container for stereo "shift" factors.
class STEREOSHIFT {
	public:

		//! Constructor.
		STEREOSHIFT (
			) : m_Scale(0), m_ZeroValue(0), m_View(STEREOVIEW_None) { }

		//! Calculate left and right points from non-displaced.
		void CalcPointDual (
			const DPOINT3D& ptBase,						//!< Base point in "device" coordinates
			DPOINT2D& ptLeft,								//!< Left point in device coordinates returned
			DPOINT2D& ptRight								//!< Right point in device coordinates returned
			) const;

		//! Calculate left view displaced point from non-displaced.
		void CalcPointLeft (
			const DPOINT3D& ptBase,						//!< Base point in "device" coordinates
			DPOINT2D& ptDisplaced						//!< Displaced point in device coordinates returned
			) const { ptDisplaced.x = ptBase.x - CalcShift(ptBase.z); ptDisplaced.y = ptBase.y; }

		//! Calculate right view displaced point from non-displaced.
		void CalcPointRight (
			const DPOINT3D& ptBase,						//!< Base point in "device" coordinates
			DPOINT2D& ptDisplaced						//!< Displaced point in device coordinates returned
			) const { ptDisplaced.x = ptBase.x + CalcShift(ptBase.z); ptDisplaced.y = ptBase.y; }

		//! Calculate left and right polylines from non-displaced.
		void CalcPolylineDual (
			const POLYLINE& plBase,						//!< Base polyline in "device" coordinates with appropriate Z value(s)
			POLYLINE& plLeft,								//!< Left polyline returned
			POLYLINE& plRight								//!< Right polyline returned
			) const;

		//! Calculate single-view displaced polyline from non-displaced for overhead stereo view.
		void CalcPolylineSingle (
			const POLYLINE& plBase,						//!< Base polyline in "device" coordinates with appropriate Z value(s)
			POLYLINE& plDisplaced						//!< Displaced polyline returned
			) const;

		//! Calculate shift for specified Z value.
		//! Must multiply result by -1 for left view shift.
		double CalcShift (
			double z
			) const { return ((m_ZeroValue - z) * m_Scale); }

		//! Get absolute scaling value.
		double GetScale () const
			{ return (m_Scale); }

		STEREOVIEW GetView () const
			{ return (m_View); }

		//! Get zero-shift value.
		double GetZeroValue () const
			{ return (m_ZeroValue); }

		//! Invalidate shift factors.
		void Invalidate ()
			{ m_Scale = 0; m_View = STEREOVIEW_None; }
		
		//! Determine if shift settings are valid.
		bool IsValid () const
			{ return (m_Scale != 0 && m_View != STEREOVIEW_None); }

		//! Set view(s) to shift and scale/zero values.
		void Set (
			STEREOVIEW View,							//!< Which view(s) to shift
			double Scale,								//!< Shift scale
			double ZeroValue							//!< Zero-shift value
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_Scale;
		double m_ZeroValue;
		STEREOVIEW m_View;
		#endif
	};

#endif	// INC_MI32_STEREOSHIFT_H
