/**
 * \file rvc/combine.h 
 * \brief interface for the RVC::COMBINE class.
 *
 * \if NODOC
 * $Id: combine.h_v 1.24 2011/12/07 22:17:37 mju Exp $
 *
 * $Log: combine.h_v $
 * Revision 1.24  2011/12/07 22:17:37  mju
 * Remove redundant hdr.
 *
 * Revision 1.23  2011/09/02 13:20:50  mju
 * Exclude macros from dox.
 *
 * Revision 1.22  2008/09/10 16:15:54  scowan
 * Added extractToShape.
 *
 * Revision 1.21  2008/09/08 13:17:50  mju
 * Fix doxygen warning.
 *
 * Revision 1.20  2006/10/13 20:38:15  mju
 * Remove rvcdefns hdr as is redundant.
 *
 * Revision 1.19  2005/05/09 17:21:07  ldyas
 * added enum to create new tables.
 *
 * Revision 1.18  2004/10/13 19:46:57  scowan
 * Added flag.
 *
 * Revision 1.17  2004/09/29 15:35:52  scowan
 * Added parameter to merge to vector.
 *
 * Revision 1.16  2004/07/16 21:51:02  scowan
 * Added Combine to vector method.
 *
 * Revision 1.15  2004/07/08 21:01:26  scowan
 * Added flag for add border.
 *
 * Revision 1.14  2004/07/02 16:11:28  scowan
 * Added a remove dup rec flag.
 *
 * Revision 1.13  2004/06/22 20:03:31  scowan
 * Replaced map proj parm with coord ref sys.
 *
 * Revision 1.12  2004/06/04 19:31:44  scowan
 * Added merge to cad.
 *
 * Revision 1.11  2004/06/03 21:11:04  scowan
 * Added parameter to extract to vector.
 *
 * Revision 1.10  2004/06/01 20:46:20  scowan
 * Added extract to region.
 *
 * Revision 1.9  2004/05/27 20:01:05  scowan
 * Added extract to cad method.
 *
 * Revision 1.8  2004/05/04 17:50:37  scowan
 * Added no validate flag.
 *
 * Revision 1.7  2004/04/28 22:04:23  scowan
 * Added combine flag.
 *
 * Revision 1.6  2004/03/25 17:13:20  scowan
 * Changed method names.
 *
 * Revision 1.5  2004/03/01 18:37:18  scowan
 * Added extract method.
 *
 * Revision 1.4  2004/02/11 20:52:47  scowan
 * Added new prototype.
 *
 * Revision 1.3  2004/02/04 14:35:40  scowan
 * Changed to use eleme select parms.
 *
 * Revision 1.2  2004/01/08 22:49:53  scowan
 * Added projection parameter.
 *
 * Revision 1.1  2004/01/07 23:51:10  scowan
 * Initial revision
 *
 *
 * \endif
**/

#ifndef INC_RVC_COMBINE_H
#define INC_RVC_COMBINE_H

#ifndef INC_RVC_VECTOR_H
	#include <rvc/vector.h>
#endif

#ifndef INC_RVC_SHAPE_H
	#include <rvc/shape.h>
#endif

namespace SPATREF {
	class COORDREFSYS;
	}

namespace RVC {

//! RVC::GEOMETRIC combination methods.
class COMBINE {
	public:
	
		enum COMBINEFLAGS {
			// Flags below match one-to-one with GEOMETRIC::EXTRACTFLAGS
			COMBINEFLAG_None =						0,
			COMBINEFLAG_Optimize =					0x00000001,		//!< Optimize object, currently only vector objects can be optimized
			COMBINEFLAG_RemExRecords =				0x00000002,		//!< Remove Excess (Unattached) database records
			COMBINEFLAG_RemStdAttTables =			0x00000004,		//!< Remove Standard Attribute database tables
			COMBINEFLAG_TableJoinNameStruct =	0x00000010,		//!< Join tables in Append functions if names alike and compatable structures
			COMBINEFLAG_TableJoinStruct =			0x00000020,		//!< Join tables in Append functions if compatable structures
			COMBINEFLAG_TableRemoveDupRec =		0x00000040,		//!< Do not copy over records if exists in dest table
			COMBINEFLAG_IgnorePolygonTables =	0x00000080,		//!< Do not copy over polygon databases
			COMBINEFLAG_RemoveTables =				0x00000100,		//!< Remove destination tables
			COMBINEFLAG_CopyHyperIndexSub =		0x00000200,		//!< Copy Hyper index subobjects
			COMBINEFLAG_DensifyPoints =			0x00000400,		//!< Densify line points using transformation parameters
			COMBINEFLAG_CreateNewTables =			0x00000800,		//!< Copy records and tables instead of maintaining links.
			// Flags below match are specific to COMBINE methods, does not match other flags
			COMBINEFLAG_MinimizeTableSize = 	  	0x10000000,		//!< Minimize table sizes (convert Identity and OneToOne to OneRecordPerElement)
			COMBINEFLAG_AddBorder =					0x20000000,		//!< Add border when clipping to a region, only useful in vector operations
			COMBINEFLAG_RemoveDupRecs =			0x40000000,		//!< Remove Duplicate Records from dest tables after all other operations except optimize
			COMBINEFLAG_NoValidate =				0x80000000		//!< Do not validate the destination object
			};
			
		enum COMBINETYPE {
			COMBINETYPE_Intersect,
			COMBINETYPE_ExclusiveUnion,
			COMBINETYPE_Union,
			COMBINETYPE_Subtract,
			COMBINETYPE_ExtractCompInside,
			COMBINETYPE_ExtractPartInside,
			COMBINETYPE_ExtractInsideBorder,
			COMBINETYPE_ExtractInsideClip,
			COMBINETYPE_ExtractCompOutside,
			COMBINETYPE_ExtractPartOutside,
			COMBINETYPE_ExtractOutsideBorder,
			COMBINETYPE_Replace
			};

		//! Combine geometric object with an operator object to a destination vector object
		static ERRVALUE CombineToVector (
			const RVC::OBJITEM& SrcObjItem,									//!< Source vector object
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SrcSelectParms,	//!< Selection of elements to extract to the destination vector object
			const RVC::OBJITEM& OpObjItem,									//!< Operator vector object
			const RVC::GEOMETRIC::ELEMSELECTPARMS& OpSelectParms,		//!< Selection of polygons to combine with the source to the destination vector object
			RVC::OBJITEM& DestObjItem,											//!< Dest object creation parms and placement of new object RETURNED
			COMBINETYPE CombineType,
			COMBINEFLAGS CombineFlags
			);
			
		//! Extract geometric object to a CAD object
		static ERRVALUE ExtractToCAD (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,	//!< Selection of elements to extract to the destination vector object
			const TRANS2D_MAPGEN& SrcToDest,
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags
			);
			
		//! Extract geometric object to a region object
		static ERRVALUE ExtractToRegion (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,	//!< Selection of elements to extract to the destination region object
			const TRANS2D_MAPGEN& SrcToDest,
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags
			);
			
		//! Extract geometric object to a shape object
		static ERRVALUE ExtractToShape (
			const OBJITEM& SrcObjItem,
			const GEOMETRIC::ELEMSELECTPARMS& SelectParms,			//!< Selection of elements to extract to the destination shape object
			const TRANS2D_MAPGEN& SrcToDest,
			SHAPE::MAKEPARMS& MakeParms,
			const DESCRIPTOR& BaseTable,
			OBJITEM& DestObjItem,											//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags
			);
			
		//! Extract geometric object to a vector object
		static ERRVALUE ExtractToVector (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,	//!< Selection of elements to extract to the destination vector object
			const TRANS2D_MAPGEN& SrcToDest,
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags,
			RVC::VECTOR::TOPOLOGYTYPE TopologyType = VECTOR::TOPOLOGYTYPE_Polygonal
			);
			
		static int LineToLineIntersection (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SrcSelect,
			const RVC::OBJITEM& OpObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& OpSelect,
			RVC::OBJITEM& DestObjItem,
			COMBINEFLAGS CombineFlags
			);
	
		//! Merge multiple geometric objects together into a CAD object
		static ERRVALUE MergeToCAD (
			const RVC::OBJITEMLIST& SrcObjItems,
			MILIST<RVC::GEOMETRIC::ELEMSELECTPARMS>& SelectList,	//!< Selection of elements to extract to the destination vector object (Must match 'SrcObjItems')
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags,
			SPATREF::COORDREFSYS* CoordRefSys = 0						//!< Set destination object projection if not NULL, otherwise use first object's projection
			);
			
		//! Merge multiple geometric objects together into a vector object
		static ERRVALUE MergeToVector (
			const RVC::OBJITEMLIST& SrcObjItems,
			MILIST<RVC::GEOMETRIC::ELEMSELECTPARMS>& SelectList,	//!< Selection of elements to extract to the destination vector object (Must match 'SrcObjItems')
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags,
			SPATREF::COORDREFSYS* CoordRefSys = 0,						//!< Set destination object projection if not NULL, otherwise use first object's projection
			RVC::VECTOR::TOPOLOGYTYPE TopologyType = VECTOR::TOPOLOGYTYPE_None		//!< Default to auto-determine the topology type, otherwise set to topology type stated
			);
			
	private:
	
		static ERRVALUE CombineNodes (
			const RVC::OBJITEM& OpObjItem,	
			const RVC::OBJITEM& DestObjItem	
			);
			
		//! Extract geometric object to a vector object
		static ERRVALUE ExtractToVector (
			const RVC::OBJITEM& SrcObjItem,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,	//!< Selection of elements to extract to the destination vector object
			const TRANS2D_MAPGEN& SrcToDest,
			RVC::OBJITEM& DestObjItem,										//!< Dest object creation parms and placement of new object RETURNED
			COMBINEFLAGS CombineFlags,
			GEOMETRIC::ELEMTRANSLIST* ElementList,
			RVC::VECTOR::TOPOLOGYTYPE TopologyType = VECTOR::TOPOLOGYTYPE_Polygonal
			);
			
		static ERRVALUE GenerateRegion (
			RVC::GEOMETRIC& GeoObj,
			const RVC::GEOMETRIC::ELEMSELECTPARMS& SelectParms,
			REGION2D& Region
			);
			
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(COMBINE::COMBINEFLAGS)
#endif

}		//! End of RVC namespace

#endif		//!< #ifndef INC_RVC_COMBINE_H
