/**
 * \file rvc/element.h
 * \brief RVC::ELEMENT class definition.
 *
 * \if NODOC
 * $Id: element.h_v 1.16 2013/01/10 16:22:02 mju Exp $
 *
 * $Log: element.h_v $
 * Revision 1.16  2013/01/10 16:22:02  mju
 * Add set method.
 *
 * Revision 1.15  2010/08/19 19:40:57  mju
 * Remove unneeded overloads.
 *
 * Revision 1.14  2010/08/19 14:47:58  mju
 * Add ctor using type only.
 * Use elementnum_None.
 *
 * Revision 1.13  2010/08/19 13:06:53  mju
 * Fix typo.
 *
 * Revision 1.12  2010/08/19 13:03:22  mju
 * Use elemnum and remove component methods as never used.
 *
 * Revision 1.11  2010/06/16 23:25:33  ldyas
 * use ifndef - instead of iffndef
 *
 * Revision 1.10  2010/06/16 21:42:35  mju
 * Allow storage of "component".
 *
 * Revision 1.9  2004/07/22 15:27:26  dwilliss
 * Added an operator< so I could use RVC::ELEMENT as a key in std::map
 *
 * Revision 1.4  2002/04/19 16:43:02  mju
 * Make ctor from elemnum explicit.
 *
 * Revision 1.1  2002/04/11 17:46:11  scowan
 * Initial revision
 * \endif
**/
 
#ifndef INC_RVC_ELEMENT_H
#define INC_RVC_ELEMENT_H

#ifndef INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif


namespace RVC {

//! Container for spatial element specifying number and ELEMTYPE.
class ELEMENT {
	public:

		//! Default constructor.	
		ELEMENT (
			) : m_Number(ELEMENTNUM_None), m_Type(ELEMTYPE_Empty) {}

		//! Construct with type only.
		explicit ELEMENT (
			ELEMTYPE ElemType
			) : m_Number(ELEMENTNUM_None), m_Type(ElemType) {}

		//! Construct from element number and optional type.
		explicit ELEMENT (
			ELEMNUM ElemNum,
			ELEMTYPE ElemType = ELEMTYPE_Empty
			) : m_Number(ElemNum), m_Type(ElemType) {}

		//! Copy construct from ELEMENT and fallback element type.
		ELEMENT (
			const ELEMENT& rhs,						//!< Element to copy
			ELEMTYPE ElemType							//!< Fallback element type, used if not set in 'rhs'
			) : m_Number(rhs.m_Number), m_Type((rhs.m_Type == ELEMTYPE_Empty) ? ElemType : rhs.m_Type) {}

		//! Clear to element info back to no element.
		void Clear (
			) { m_Number = ELEMENTNUM_None; m_Type = ELEMTYPE_Empty; }

		//! Get element number.
		ELEMNUM GetNumber (
			) const { return (m_Number); }
			
		//! Get element type.
		ELEMTYPE GetType (
			) const { return (m_Type); }

		//! Set type and number
		void Set (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum
			) { m_Type = ElemType; m_Number = ElemNum; }

		//! Set element number.
		void SetNumber (
			ELEMNUM ElemNum
			) { m_Number = ElemNum; }

		//! Set element type (ELEMTYPE).
		void SetType (
			ELEMTYPE ElemType
			) { m_Type = ElemType; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMNUM m_Number;
		ELEMTYPE m_Type;			// Would only need to be UINT8 so if add more members take from here
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};

//! Compare ELEMENTs for equality.
inline bool operator== (
	const ELEMENT& lhs,
	const ELEMENT& rhs
	) { return (lhs.GetNumber() == rhs.GetNumber() && lhs.GetType() == rhs.GetType()); }
	
//! Compare ELEMENTs for inequality.
inline bool operator!= (
	const ELEMENT& lhs,
	const ELEMENT& rhs
	) { return (!operator==(lhs,rhs)); }
	
//! Compare ELEMENTs for ordering.
//! Elements are sorted by type then by element number.
//! This comparison allows RVC::ELEMENT to be used as a Key in std::map.
inline bool operator< (
	const ELEMENT& lhs,
	const ELEMENT& rhs
	) {
	if (lhs.GetType() == rhs.GetType()) return (lhs.GetNumber() < rhs.GetNumber());
	return (static_cast<int>(lhs.GetType()) < static_cast<int>(rhs.GetType()));
	}
	
}		// End of RVC namespace

#endif 	// INC_RVC_ELEMENT_H

