/**
 * \file rvc/opcontouring.h
 * \brief Definitions for RVC::OP::CONTOURING class.
 *
 * \if NODOC
 * $Id: opcontouring.h_v 1.8 2009/06/26 12:51:01 mju Exp $
 *
 * $Log: opcontouring.h_v $
 * Revision 1.8  2009/06/26 12:51:01  mju
 * Use op::component class.
 *
 * Revision 1.7  2009/06/24 19:11:05  mju
 * Clean up and make some accessors inline.
 *
 * Revision 1.5  2007/12/13 17:37:14  vdronov
 * fixed SetResolutionFactor
 *
 * Revision 1.1  2007/01/12 16:41:38  vdronov
 * Initial revision
 * \endif
**/

#ifndef  INC_RVC_CONTOURING_H
#define  INC_RVC_CONTOURING_H

#ifndef	INC_RVC_OPCOMPONENT_H
	#include <rvc/opcomponent.h>
#endif

#ifndef INC_RVC_VECTOR_H
	#include <rvc/vector.h>
#endif

#ifndef INC_MI32_QTREE_H
	#include <mi32/qtree.h>
#endif

#include <float.h>


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
namespace SPATREF {
	class COORDREFSYS;
	}
#endif


namespace RVC {
namespace OP {

//=====================================================================================================================

//! Class for creating contour lines.
class CONTOURING {
	public:

	   //! Available sources for contouring.
		enum SOURCE {
			SOURCE_Invalid = -1,
			SOURCE_First = 0,
			SOURCE_Raster = 0,
			SOURCE_TIN = 1,
			SOURCE_Last = 1,
			SOURCE_Count = 2
			};

	   //! Methods of contouring
		enum METHOD {
			METHOD_Invalid = -1,
			METHOD_First = 0,
			METHOD_Linear = 0,     
			METHOD_IterativeThresholding = 1,  
			METHOD_Last = 1,
			METHOD_Count = 2
			};

		//! Class describes parameters of source object.
		class SOURCE_PARAMETERS {
			public:

				SOURCE_PARAMETERS (
					const SOURCE source
					);

				virtual ~SOURCE_PARAMETERS (
					);

				SOURCE GetSource (
					) const { return m_Source; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				const SOURCE m_Source;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of Raster source object.
		class RASTER_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				//! enum describes resampling methods.
				enum RESAMPLINGMETHOD {
					RESAMPLINGMETHOD_None = 0,
					RESAMPLINGMETHOD_BilinearInterpolation = 1,
					RESAMPLINGMETHOD_CubicConvolution = 2
					};

				//! enum describes smoothing methods.
				enum SMOOTHINGMETHOD {
					SMOOTHINGMETHOD_None = 0,
					SMOOTHINGMETHOD_WeightedAverage = 1,
					SMOOTHINGMETHOD_Gaussian = 2,
					SMOOTHINGMETHOD_Quadratic = 3,
					SMOOTHINGMETHOD_Cubic = 4,
					SMOOTHINGMETHOD_Quartic = 5
					};

				//! enum describes window size needed for some smoothing methods.
				enum WINDOWSIZE {
					WINDOWSIZE_3x3 = 3,
					WINDOWSIZE_5x5 = 5,
					WINDOWSIZE_7x7 = 7,
					WINDOWSIZE_9x9 = 9,
					WINDOWSIZE_11x11 = 11,
					WINDOWSIZE_13x13 = 13,
					WINDOWSIZE_15x15 = 15
					};

				RASTER_PARAMETERS (
					);

				~RASTER_PARAMETERS (
					);

				//! Returns resampling method.
				RESAMPLINGMETHOD GetResamplingMethod (
					) const { return m_ResamplingMethod; };

				//! Returns resolution factor for resampling.
				UINT8 GetResolutionFactor (
					) const { return m_ResolutionFactor; } 

				//! Returns smoothing method.
				SMOOTHINGMETHOD GetSmoothingMethod (
					) const { return m_SmoothingMethod; };

				//! Returns window size for smoothing.
				WINDOWSIZE GetWindowSize (
					) const { return m_WindowSize; };

				//! Set resampling method.
				void SetResamplingMethod (
					const RESAMPLINGMETHOD method
					) { m_ResamplingMethod = method; };

				//! Set resolution factor for resampling.
				UINT8 SetResolutionFactor (
					const UINT8 ResolutionFactor
					) { 
					const UINT8 factor1 = ResolutionFactor >> 1;
					const UINT8 factor2 = bound(factor1, 1, 8);
					m_ResolutionFactor =  factor2 << 1;  
					return m_ResolutionFactor; 
					} 

				//! Set smoothing method.
				void SetSmoothingMethod (
					const SMOOTHINGMETHOD method
					) { m_SmoothingMethod = method; };

				//! Set window size for smoothing.
				void SetWindowSize (
					const WINDOWSIZE size
					) { m_WindowSize = size; };

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				RESAMPLINGMETHOD m_ResamplingMethod;
				UINT8 m_ResolutionFactor;
				SMOOTHINGMETHOD m_SmoothingMethod;
				WINDOWSIZE m_WindowSize;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Class describes parameters of TIN source object.
		class TIN_PARAMETERS : public SOURCE_PARAMETERS {
			public:

				TIN_PARAMETERS (
					);

				~TIN_PARAMETERS (
					);

				//! Get component for accessing selection and valuation settings.
				const OP::COMPONENT& GetComponent (
					) const { return m_Component; };

				//! Set component selection and valuation.
				void SetComponent (
					const OP::COMPONENT& element
					) { m_Component = element; };

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				OP::COMPONENT m_Component;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class describes parameters of destination object.
		class DESTINATION_PARAMETERS {
			public:

				DESTINATION_PARAMETERS (
					);

				~DESTINATION_PARAMETERS (
					);

				//! Return true if closing of countour lines and creating of Elevation Range table has to be performed.
				bool GetCreateElevationRangeTable (
					) const { return m_CreateElevationRangeTable; }

				//! Set to true if closing of countour lines and creating of Elevation Range table has to be performed.
				void SetCreateElevationRangeTable (
					const bool create
					) { m_CreateElevationRangeTable = create; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				bool m_CreateElevationRangeTable;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Base class for all contouring methods.
		class METHOD_PARAMETERS {
			public:

				METHOD_PARAMETERS (
					const METHOD method
					);

				virtual ~METHOD_PARAMETERS (
					);

				//! Return method of contouring
				METHOD GetMethod (
					) const { return m_Method; }

				//! Return starting level of contouring
				double GetStartingLevel (
					) const { return m_StartingLevel; }

				//! Return ending level of contouring
				double GetEndingLevel (
					) const { return m_EndingLevel; }

				//! Return interval of contouring
				double GetInterval (
					) const { return m_Interval; }

				//! Return z scale of source elements
				double GetZScale (
					) const { return m_ZScale; }

				//! Return z offset of source elements
				double GetZOffset (
					) const { return m_ZOffset; }

				//! Set starting level of contouring
				void SetStartingLevel (
					const double StartingLevel
					) { m_StartingLevel = StartingLevel; }

				//! Set ending level of contouring
				void SetEndingLevel (
					const double EndingLevel
					) {m_EndingLevel = EndingLevel; }

				//! Set interval of contouring
				void SetInterval (
					const double Interval
					) { m_Interval = Interval; }

				//! Set z scale of source elements
				void SetZScale (
					const double ZScale
					) { m_ZScale = ZScale; }

				//! Set z offset of source elements
				void SetZOffset (
					const double ZOffset
					) { m_ZOffset = ZOffset; } 

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT

				const METHOD m_Method;

				double m_StartingLevel;
				double m_EndingLevel;
				double m_Interval;
				double m_ZScale;
				double m_ZOffset;

				METHOD_PARAMETERS (
					); // not implemented

				static METHOD Check (
					const METHOD method
					);

				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Class for Linear contouring method from TIN and Raster.
		class LINEAR_PARAMETERS : public METHOD_PARAMETERS {
			public:

				LINEAR_PARAMETERS (
					) :
					METHOD_PARAMETERS(METHOD_Linear)
					{ } 

				~LINEAR_PARAMETERS (
					) { }

			};

		//! Class for Iterative Thresholding contouring method from  Raster.
		class ITERATIVETHRESHOLDING_PARAMETERS : public METHOD_PARAMETERS {
			public:

				ITERATIVETHRESHOLDING_PARAMETERS (
					)  :
					METHOD_PARAMETERS(METHOD_IterativeThresholding)
					{}

				~ITERATIVETHRESHOLDING_PARAMETERS (
					) {}

			};

		//! Class for collecting and tracing segments to polylines following to saving them to vector.
		class SEGMENTS {
			public:

				// Constructor.
				SEGMENTS ();

				~SEGMENTS ();

				//! Add segment 
				ERRVALUE Add (
					const DPOINT3D& p1,
					const DPOINT3D& p2
					);

				//! Clear all buffers and close all objects.
				void Clear (
					);

				//! Init with existing vector objitem and its extents.
				ERRVALUE Init (
					const RVC::OBJITEM& objitem,
					const DRECT3D& extents
					);

				//! Trace segments if their number greater than 16K 
				ERRVALUE Trace (
					const bool force = false //! if true force to trace unconditionally
					);

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT
				struct SEGMENT {
					DPOINT3D m_Left;
					DPOINT3D m_Right;
					bool m_IsTheSameZ;
					bool m_IsUsed;		

					SEGMENT (
						) :
						m_IsTheSameZ(true),
						m_IsUsed(false)
						{ }
					};

				RVC::VECTOR m_Vector;
				QUADTREE m_QTree;
				POLYLINE m_Polyline;
				POLYLINE m_Baseline;
				POLYLINE m_Joinline;
				POLYLINE m_Templine;
				SIMPLE_ARRAY<SEGMENT> m_Segments;
				SIMPLE_ARRAY<INT32> m_ListA;
				SIMPLE_ARRAY<INT32> m_ListB;

				double m_Tolerance;

				static int Compare (SEGMENT* p1, SEGMENT* p2, void* );
				static double Distance (const DPOINT2D& p1, const DPOINT2D& p2);

				ERRVALUE ReplaceNode (const DPOINT2D& op, const DPOINT2D& np, const INT32 itemnum);
				int Snap (POLYLINE& polyline);
				ERRVALUE Store (POLYLINE& polyline);
				ERRVALUE StoreSimple (POLYLINE& polyline, const INT32 linenum);

				#endif // GENERATING_DOXYGEN_OUTPUT

			};


		//! Generates contouring
		static ERRVALUE Process (
		   const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const SOURCE_PARAMETERS* SrcParameters,
			const METHOD_PARAMETERS* MethodParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

	   //! Generates contouring from TIN
	   static ERRVALUE ProcessLinearFromTIN (
	      const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const TIN_PARAMETERS& TINParameters,
			const LINEAR_PARAMETERS& LinearParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

	   //! Generates contouring from raster
	   static ERRVALUE ProcessFromRaster (
	      const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const RASTER_PARAMETERS& RasterParameters,
			const METHOD_PARAMETERS* MethodParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

	   //! Generates contouring from raster by Linear
	   static ERRVALUE ProcessLinearFromRaster (
	      const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const RASTER_PARAMETERS& RasterParameters,
			const LINEAR_PARAMETERS& LinearParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

	   //! Generates contouring from raster by Iterative Thresholding
	   static ERRVALUE ProcessIterativeThresholdingFromRaster (
	      const RVC::OBJITEM& SrcObjItem,
	      RVC::OBJITEM& DestObjItem, 
			const RASTER_PARAMETERS& RasterParameters,
			const ITERATIVETHRESHOLDING_PARAMETERS& IterThresParameters,
			const DESTINATION_PARAMETERS& DestParameters 
			);

		//! Compute full set of contour segments for a given triangle (as a set of 3 nodes).
		static ERRVALUE ComputeFromTriangle (
			const DPOINT3D& A,								//!< Node of the triangle
			const DPOINT3D& B,								//!< Node of the triangle
			const DPOINT3D& C,								//!< Node of the triangle
			double start,										//!< Starting level of contouring range (Z)
			double end,											//!< Ending level of contouring range (Z)
			double interval,									//!< Interval between contour lines in Z units
			FastDelegate<ERRVALUE(const DPOINT3D&, const DPOINT3D&)> DelegateContourSegment	//! Delegate that will be envoked for each contour segment
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
	   static ERRVALUE ProcessDestination (
	      const RVC::OBJITEM& DestObjItem, 
			const DESTINATION_PARAMETERS& DestParameters 
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	       
	};

//=====================================================================================================================

}  // end namespace OP
}  // end namespace RVC


#endif  // INC_RVC_OPPROFILING_H
