/**
 * \file rvc/rastersensor.h 
 * \brief interface for the RVC::RASTERSENSOR class.
 *
 * \if NODOC
 * $Id: rastersensor.h_v 1.4 2007/09/06 17:18:54 scowan Exp $
 *
 * $Log: rastersensor.h_v $
 * Revision 1.4  2007/09/06 17:18:54  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.3  2006/07/26 20:56:12  scowan
 * Added virtual methgod override.
 *
 * Revision 1.2  2006/03/03 20:38:09  scowan
 * Added more methods.
 *
 * Revision 1.1  2006/03/03 18:49:51  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::RASTERSENSOR rvc/rastersensor.h
//!	\brief Manages RVC::OBJTYPE_SensorInfo RVC objects.
//!
//!	The interface currently wraps the MfSpectralInfo functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_SensorInfo
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_SensorInfo
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!
//!	Read(), Make() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods


#ifndef INC_RVC_RASTERSENSOR_H
#define INC_RVC_RASTERSENSOR_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class DATETIME;
#endif // GENERATING_DOXYGEN_OUTPUT


namespace RVC {

class RASTERSENSOR : public RVC::SIMPLE {
	public:
		enum CALIBTYPE {	
			CALIBTYPE_FlatField = 1,		//!< Simple flat field calibration 
			CALIBTYPE_EqualArea_IAR = 2,	//!< Equal area calibration using Internal Average Reflectance spectrum 
			CALIBTYPE_LogResiduals = 3		//!< Logarithmic Residuals tecnique 
			};
			
		enum CURVETYPE {
			CURVETYPE_Gaussian = 0,		//!< Wavelength and bandwidth defines Gaussian curve, hyperspectral images 
			CURVETYPE_NonGaussian = 1	//!< Spectral sensitivity is not gaussian and can't be derived from wavelength and bandwidth fields 
			};

		enum SENSOR {		//! Some initial sensor definitions, this field is just for information only.
			SENSOR_Unknown = 0,
			SENSOR_AVIRIS = 1,
			SENSOR_PROBE1 = 2,
			SENSOR_CASI = 3,
			SENSOR_HYMAP = 4,
			SENSOR_DIAS = 5,
			SENSOR_ORBVIEW4 = 6
			};

		RASTERSENSOR (
			);

		RASTERSENSOR (
			const RVC::RASTERSENSOR& rhs
			);

		virtual ~RASTERSENSOR (
			);

		RVC::RASTERSENSOR& operator= (
			const RVC::RASTERSENSOR& rhs
			);

		double GetAltitude (
			) const { return (m_altitude); }

		double GetAtmosphereThickness (
			) const { return (m_athmthickness); }
		
		INT32 GetBand (
			) const { return (m_band); }
			
		double GetBandwidth (
			) const { return (m_bandwidth); }

		double GetBlackFieldRadiance (
			) const { return (m_blackfieldradiance); }
		
		CALIBTYPE GetCalibrationType (			
			) const { return (m_calibrationtype); }

		CURVETYPE GetCurveType (
			) const { return (m_curvetype); }

		double GetFlatFieldRadiance (
			) const { return (m_flatfieldradiance); }

		void GetImageDate (
			DATETIME& ImageDate
			) const;
			
		SENSOR GetSensorType (
			) const { return (m_sensor); }

		double GetSignalError (
			) const { return (m_signalerror); }
			
		double GetSignalNoise (
			) const { return (m_signalnoise); }
			
		double GetWavelength (
			) const { return (m_wavelength); }

		double GetSunAngle (
			) const { return (m_sunangle); }

		void SetAltitude (
			double Altitude
			) { m_altitude = Altitude; }

		void SetAtmosphereThickness (
			double AtmosphereThickness			//!< Equivalent athmospheric thickness through which the observation was obtained 
			) { m_athmthickness = AtmosphereThickness; }
		
		void SetBand (
			INT32 Band
			) { m_band = Band; }
			
		void SetBandwidth (
			double Bandwidth
			) { m_bandwidth = Bandwidth; }

		void SetBlackFieldRadiance (
			double BlackFieldRadiance			//!< Needed for compensation of extra signal and converting to "apparent reflectance"
			) { m_blackfieldradiance = BlackFieldRadiance; }
		
		void SetCalibrationType (			
			CALIBTYPE CalibrationType
			) { m_calibrationtype = CalibrationType; }

		void SetCurveType (
			CURVETYPE CurveType
			) { m_curvetype = CurveType; }

		void SetFlatFieldRadiance (
			double FlatFieldRadiance			//!< Needed for converting image pixels to "apparent reflectance"
			) { m_flatfieldradiance = FlatFieldRadiance; }

		void SetImageDate (
			const DATETIME& ImageDate
			);

		void SetSensorType (
			SENSOR sensor
			) { m_sensor = sensor; }

		void SetSignalError (
			double signalerror						//!< RMS error of the signal value in this image 
			) { m_signalerror = signalerror; }
			
		void SetSignalNoise (
			double signalnoise						//!< Signal/noise ratio 
			) { m_signalnoise = signalnoise; }
			
		void SetWavelength (
			double Wavelength
			) { m_wavelength = Wavelength; }

		void SetSunAngle (
			double SunAngle							//!< Sun angle at the moment of the survey, deg 
			) { m_sunangle = SunAngle; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVCDATETIME	m_imagedate;				//!< Time when image was taken 
		CURVETYPE m_curvetype;					//!< SPECTRALCURVE_GAUSSIAN - for hyper spectral data, NON_GAUSSIAN - for other data 
		CALIBTYPE m_calibrationtype;			//!< Calibration type SPECTRALCALIB_ ... that was done on this image 
		SENSOR m_sensor;							//!< SENSOR_ constant 
		INT32 m_band;								//!< Band
		UINT32 m_flags;							//!< Flags
		double m_wavelength;						//!< Wavelength in mkm 
		double m_bandwidth;			 			//!< Bandwidth in mkm 
		double m_altitude;			 			//!< Sensor altitude at the moment of the survey, km 
		double m_sunangle;			 			//!< Sun angle at the moment of the survey, deg 
		double m_athmthickness;					//!< Equivalent athmospheric thickness through which the observation was obtained 
		double m_signalnoise;		 			//!< Signal/noise ratio 
		double m_signalerror;		 			//!< RMS error of the signal value in this image 
		double m_flatfieldradiance; 			//!< Needed for converting image pixels to "apparent reflectance"
		double m_blackfieldradiance;			//!< Needed for compensation of extra signal and converting to "apparent reflectance"
		
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_ValidateObjectType (const RVC::OBJTYPE otype) const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_RASTERSENSOR_H
