/**
 * \file imagesection.h <spatmod/imagesection.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::BASE class definitions
 *
 * \if NODOC
 * $Id: imagesection.h_v 1.16 2011/03/31 20:42:20 mju Exp $
 *
 * $Log: imagesection.h_v $
 * Revision 1.16  2011/03/31 20:42:20  mju
 * Add getLogString.
 *
 * Revision 1.15  2011/02/03 16:50:23  mju
 * Add getMinX/Y.
 *
 * Revision 1.14  2008/08/28 14:00:37  mju
 * Fix doxygen warning.
 *
 * Revision 1.13  2007/04/24 15:45:49  mju
 * Make computeMinColumn/Row non-inline.
 *
 * Revision 1.12  2007/01/04 17:16:57  mju
 * Add setMin.
 *
 * Revision 1.11  2006/11/15 15:04:59  mju
 * Add TestEfficiency.
 *
 * Revision 1.10  2006/11/10 14:21:35  mju
 * Add begin method that uses spans.
 *
 * Revision 1.9  2006/11/02 15:44:04  mju
 * Use spans class.
 *
 * Revision 1.8  2006/11/01 23:11:14  mju
 * quryIterator.
 *
 * Revision 1.7  2006/10/16 19:42:12  mju
 * Exclude enum op from doxygen.
 *
 * Revision 1.6  2006/10/11 17:35:48  mju
 * Add computeRect.
 *
 * Revision 1.5  2006/07/28 17:31:45  mju
 * Add setting for irregular iterator.
 *
 * Revision 1.4  2005/09/06 14:42:23  mju
 * Include math hdr.
 *
 * Revision 1.3  2005/08/19 14:25:59  mju
 * Add iterator.getNumRows/ColumnsOfTiles.
 *
 * Revision 1.2  2005/08/17 14:36:21  mju
 * Forgot bool on inline operators.
 *
 * Revision 1.1  2005/08/16 15:10:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESECTION_H
#define	INC_SPATMOD_IMAGESECTION_H

#ifndef	INC_SPATMOD_IMAGESPANS_H
#include <spatmod/imagespans.h>
#endif

#ifndef	INC_MI32_MATH_H
#include <mi32/math.h>
#endif

struct DRECT2D;
struct LRECT2D;

namespace SPATMOD {
namespace IMAGE {

class BASE;
class DIMENSIONS;
class SECTION_ITERATOR;

//=====================================================================================================================
//! Container for specifying rectangular section of image.
class SECTION {
	public:

		//! Section position when used from SECTION_ITERATOR.
		enum POSITION {
			POSITION_Unknown =					0,
			POSITION_TileColumnFirst =			0x0001,
			POSITION_TileColumnInterior =		0x0002,
			POSITION_TileColumnLast =			0x0004,
			POSITION_TileRowFirst =				0x0010,
			POSITION_TileRowInterior =			0x0020,
			POSITION_TileRowLast =				0x0040,
			POSITION_StripFirst =				0x0100,
			POSITION_StripInterior =			0x0200,
			POSITION_StripLast =					0x0400,
			};

		//! Default contructor, creates invalid/empty section.
		SECTION ();

		//! Construct section for entire image (implicit).
		//! Will construct invalid section if image does not support specified zoom value
		SECTION (
			const BASE& image,
			double zoom = 1
			);

		//! Construct using DRECT2D.
		explicit SECTION (
			const DRECT2D& rect,
			double zoom = 1
			);

		//! Construct using LRECT2D.
		explicit SECTION (
			const LRECT2D& rect,
			double zoom = 1
			);

		//! Construct using specified values.
		SECTION (
			double MinX,
			double MinY,
			double SizeX,
			double SizeY,
			double zoom = 1,
			POSITION position = POSITION_Unknown
			);

		//! Compute maximum image column.
		//! If maximum X position is not an integer this will be the column containing
		//! the maximum X position.
		INT32 ComputeMaxColumn (
			) const;

		//! Compute maximum image row.
		//! If maximum Y position is not an integer this will be the row containing
		//! the maximum Y position.
		INT32 ComputeMaxRow (
			) const;

		//! Compute minimum image column.
		//! If minimum X position is not an integer this will be the column containing
		//! the minimum X position.
		INT32 ComputeMinColumn (
			) const;

		//! Compute minimum image row.
		//! If minimum Y position is not an integer this will be the row containing
		//! the minimum Y position.
		INT32 ComputeMinRow (
			) const;

		//! Compute number of whole or partial columns in section.
		//! Equivalent to ComputeMaxColumn() - ComputeMinColumn() + 1.
		INT32 ComputeNumColumns (
			) const;

		//! Compute number of whole or partial rows in section.
		//! Equivalent to ComputeMaxRow() - ComputeMinRow() + 1.
		INT32 ComputeNumRows (
			) const;

		//! Compute rectangle covering all whole or partial rows/columns.
		void ComputeRect (
			LRECT2D& rect					//!< Rectangle returned
			) const;

		//! Get string for use in log.
		MISTRING GetLogString (
			bool showzoom = true
			) const;

		//! Get maximum X position.
		//! Equivalent to GetMinX() + GetSizeX().
		double GetMaxX (
			) const { return (m_MinX + m_SizeX); }

		//! Get maximum Y position.
		//! Equivalent to GetMinY() + GetSizeY().
		double GetMaxY (
			) const { return (m_MinY + m_SizeY); }

		//! Get minimum X position.
		double GetMinX (
			) const { return (m_MinX); }

		//! Get minimum Y position.
		double GetMinY (
			) const { return (m_MinY); }

		//! Get position as set by SECTION_ITERATOR.
		POSITION GetPosition (
			) const { return (m_Position); }

		//! Get rectangle as X/Y ranges.
		void GetRect (
			DRECT2D& rect					//!< Rectangle returned
			) const;

		//! Get X size of section.
		double GetSizeX (
			) const { return (m_SizeX); }

		//! Get Y size of section.
		double GetSizeY (
			) const { return (m_SizeY); }

		//! Get zoom factor.
		double GetZoom (
			) const { return (m_Zoom); }

		//! Compare for equality.
		bool IsEqual (
			const SECTION& rhs
			) const;

		//! Set minimum X/Y values.
		void SetMin (
			double MinX,
			double MinY
			) { m_MinX = MinX; m_MinY = MinY; }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		double m_MinX;
		double m_MinY;
		double m_SizeX;
		double m_SizeY;
		double m_Zoom;
		POSITION m_Position;
		friend class SECTION_ITERATOR;
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SECTION::POSITION);
#endif

//! Compare for equality.
inline bool operator== (
	const SECTION& lhs,
	const SECTION& rhs
	) { return (lhs.IsEqual(rhs)); }

//! Compare for inequality.
inline bool operator!= (
	const SECTION& lhs,
	const SECTION& rhs
	) { return (!lhs.IsEqual(rhs)); }


//=====================================================================================================================

//! Iterator to process a specified section in subsections.
//! Can iterate by strips, tiles or strips within tiles.
//! When iterating by tiles, all tiles in a tile-row are processed before
//! any tiles in the next tile-row.  If tiles are further divided by strips
//! all strips in one tile are processed before moving to the next tile.
//! It is up to the caller to ensure that the subsection size is small enough to fit
//! in a memory buffer when eventually used for read/write access.
class SECTION_ITERATOR {
	public:

		//! Construct empty iterator.
		SECTION_ITERATOR ();

		//! Destructor.
		~SECTION_ITERATOR ();

		//! Dereference operator, returns reference to SECTION
		const SECTION& operator* (
			) const { return (m_CurSection); }

		//! Dereference operator, returns pointer to SECTION
		const SECTION* operator-> (
			) const { return (&m_CurSection); }

		//! Pre-increment operator.
		SECTION_ITERATOR& operator++ (
			);

		//! Start iterator with values for strip sequence.
		ERRVALUE Begin (
			const SECTION& TotalSection,			//!< Total section to iterate through
			double StripSizeY							//!< Size of strip
			);

		//! Start iterator with values for tile or tile/strip sequence.
		ERRVALUE Begin (
			const SECTION& TotalSection,			//!< Total section to iterate through
			const SPANS& SpansX,						//!< X spans, size must match total section
			const SPANS& SpansY,						//!< Y spans, size must match total section
			double StripSizeY = 0,					//!< Size of strip, 0 if processing whole tiles
			double FirstStripSizeY = 0				//!< Y size of first strip if different, 0 if same
			);

		//! Start iterator with values for tile or tile/strip sequence.
		ERRVALUE Begin (
			const SECTION& TotalSection,			//!< Total section to iterate through
			double StripSizeY,						//!< Size of strip, 0 if processing whole tiles
			double InteriorTileSizeX,				//!< X tile size
			double InteriorTileSizeY,				//!< Y tile size
			double FirstTileSizeX = 0,				//!< X size for first tile if different, 0 if same
			double FirstTileSizeY = 0,				//!< Y size for first tile if different, 0 if same
			double LastTileSizeX = 0,				//!< X size for last tile if different, 0 to compute
			double LastTileSizeY = 0,				//!< Y size for last tile if different, 0 to compute
			double FirstStripSizeY = 0				//!< Y size of first strip if different, 0 if same
			);

		//! Start iterator with tile properties obtained from image.
		//! Will also determine proper alignment of tiles if total section does not cover entire image
		ERRVALUE Begin (
			const SECTION& TotalSection,			//!< Total section to iterate through
			const BASE& Image,						//!< Image to obtain tile properties from
			double StripSizeY = 0					//!< Size of strip, 0 if processing whole tiles
			);

		//! Start iterator with optimal sequence based on source and target image capabilities.
		//! Source and target image must have same total dimensions.
		ERRVALUE Begin (
			const SECTION& TotalSection,			//!< Total section to iterate through
			const BASE& TargetImage,				//!< Target image
			const BASE& SourceImage,				//!< Source image
			double StripSizeY = 0					//!< Size of strip, 0 for default based on tile size and sequence
			);

		//! Restart iterator at beginning using previous section settings.
		void Begin (
			);

		//! Compute maximum subsection columns.
		INT32 ComputeMaxSubSectionColumns (
			) const;

		//! Compute maximum subsection rows.
		INT32 ComputeMaxSubSectionRows (
			) const;

		//! Compute maximum subsection X size.
		double ComputeMaxSubSectionSizeX (
			) const;

		//! Compute maximum subsection Y size.
		double ComputeMaxSubSectionSizeY (
			) const;

		//! Compute total number of iterations.
		INT32 ComputeNumIterations (
			) const;

		//! Get Y size of first strip.
		double GetFirstStripSizeY (
			) const { return (m_FirstStripSizeY); }

		//! Get X size of first tile.
		double GetFirstTileSizeX (
			) const { return (m_SpansX.GetSpanSizeFirst()); }

		//! Get Y size of first tile.
		double GetFirstTileSizeY (
			) const { return (m_SpansY.GetSpanSizeFirst()); }

		//! Get X size of interior tile.
		double GetInteriorTileSizeX (
			) const { return (m_SpansX.GetSpanSizeInterior()); }

		//! Get Y size of interior tile.
		double GetInteriorTileSizeY (
			) const { return (m_SpansY.GetSpanSizeInterior()); }

		//! Get X size of last tile.
		double GetLastTileSizeX (
			) const { return (m_SpansX.GetSpanSizeLast()); }

		//! Get Y size of last tile.
		double GetLastTileSizeY (
			) const { return (m_SpansY.GetSpanSizeLast()); }

		//! Get minimum X position.
		double GetMinX () const
			{ return (m_MinX); }

		//! Get minimum Y position.
		double GetMinY () const
			{ return (m_MinY); }

		//! Get number of columns of tiles.
		UINT32 GetNumColumnsOfTiles (
			) const { return (m_SpansX.GetNumSpans()); }

		//! Get number of rows of tiles.
		UINT32 GetNumRowsOfTiles (
			) const { return (m_SpansY.GetNumSpans()); }

		//! Get X span values.
		const SPANS& GetSpansX (
			) const { return (m_SpansX); }

		//! Get Y span values.
		const SPANS& GetSpansY (
			) const { return (m_SpansY); }

		//! Get strip Y size.
		double GetStripSizeY (
			) const { return (m_StripSizeY); }

		//! Get total section which was specified in Begin().
		const SECTION& GetTotalSection (
			) const { return (m_TotSection); }

		//! Check if at end.
		bool IsEnd (
			) const { return (m_CurTileRow >= m_SpansY.GetNumSpans()); }

		//! Determine if iterator is irregular.
		bool IsIrregular (
			) const { return (m_IsIrregular); }

		//! Determine if iterator tiles align with image dimensions.
		bool IsTileAligned (
			const DIMENSIONS& dimensions			//!< Image dimensions
			) const;

		//! Set iterator as irregular.
		//! An irregular iterator cannot does not support simple increment, the owner
		//! is responsible for computing the actual section needed at each iteration.
		void SetIrregular (
			const SECTION& TotalSection,			//!< Total section to iterate through
			UINT32 MaxColumns,						//!< Maximum columns
			UINT32 MaxRows								//!< Maximum rows
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		SECTION m_TotSection;
		SECTION m_CurSection;
		SPANS m_SpansX;
		SPANS m_SpansY;
		double m_MinX;
		double m_MinY;
		double m_StripSizeY;
		double m_FirstStripSizeY;
		double m_CurTileSizeY;
		UINT32 m_CurTileRow;
		UINT32 m_CurTileColumn;
		UINT32 m_CurTileNumStrips;
		UINT32 m_CurStripRow;
		bool m_IsIrregular;

		void ComputeCurSection ();
		void ComputeCurTile ();
		void Setup (
			const SECTION& TotalSection,
			double StripSizeY,
			double InteriorTileSizeX,
			double InteriorTileSizeY,
			double FirstTileSizeX,
			double FirstTileSizeY
			);
		bool TestEfficiency (
			const BASE& TargetImage,
			const BASE& SourceImage,
			float& BestEfficiency,
			SECTION_ITERATOR& BestIterator
			) const;

		#endif	// GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGESECTION_H
